

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>



#include "../src/defs.h"
#include "../src/dlgbox.h"
#include "../src/functions.h"
#include "../src/findclust.h"
#include "../src/interface.h"
#include "../src/graphdsp.h"
#include "../src/dso.h"
#include "../src/macro.h"
#include "../src/gtkhelpers.h"
#include "../src/special1.h"
#include "../src/imgbuf.h"
#include "../src/application.h"
#include "../src/secondwnd.h"
#include "../src/tiffstuff.h"
#include "../src/roi.h"
#include "../src/viz.h"


/* Data needed to compute & run cubic splines through the vertices 
	-  see Dan Olfe, pg. 169 */


typedef struct
{
	double gamma;	/* The matrix */
	double delta;	/* the RHS vector */
	double dee;		/* The first derivatives */
	double c;		/* pwr-of-2 coefficient */
	double d;		/* pwr-of-3 coefficient */
	double v;		/* The actual vertex */

}
splineset;

static splineset *splx, *sply;
static int spline_prep = 0;
static vertex pv;
int *xdata, *ydata, *hdata;
int pixelcnt;


void recompute_spline ()
{
int i,n;

	dp (1, "Trailmaker: Recomputing spline coefficients\n");

	/* Allocate spline vertex memory for the current number of ROI vertices */
	/* Needs to be freed in window_close or we have a huge memory leak */

	splx = calloc (current_roi.numvertices+10, sizeof (splineset));		/* x(t,k) */
	sply = calloc (current_roi.numvertices+10, sizeof (splineset));		/* y(t,k) */
	if ((!sply) || (!splx)) fatal_memerr ();

	/* Fill the vertex data, compute gamma at the same time */

	n = current_roi.numvertices;
	splx[0].gamma = sply[0].gamma = 0.5;
	splx[0].v = current_roi.xi[0];
	sply[0].v = current_roi.yi[0];
	splx[n].v = current_roi.xi[0];
	sply[n].v = current_roi.yi[0];
	for (i=1; i<n; i++)
	{
		splx[i].v = current_roi.xi[i];
		sply[i].v = current_roi.yi[i];
		splx[i].gamma = 1.0/(4.0-splx[i-1].gamma);
		sply[i].gamma = 1.0/(4.0-sply[i-1].gamma);
	}

	splx[n].gamma = 1.0/(2.0-splx[n-1].gamma);
	sply[n].gamma = 1.0/(2.0-sply[n-1].gamma);
	splx[0].delta = 3.0*(splx[1].v-splx[0].v)*splx[0].gamma;
	sply[0].delta = 3.0*(sply[1].v-sply[0].v)*sply[0].gamma;

	for (i=1; i<n; i++)
	{
		splx[i].delta = ( 3*(splx[i+1].v-splx[i-1].v)-splx[i-1].delta ) * splx[i].gamma;
		sply[i].delta = ( 3*(sply[i+1].v-sply[i-1].v)-sply[i-1].delta ) * sply[i].gamma;
	}
	splx[n].delta = ( 3*(splx[n].v-splx[n-1].v)-splx[n-1].delta ) * splx[n].gamma;
	sply[n].delta = ( 3*(sply[n].v-sply[n-1].v)-sply[n-1].delta ) * sply[n].gamma;
	splx[n].dee = splx[n].delta;
	sply[n].dee = sply[n].delta;

	for (i=n-1; i>=0; i--)
	{
		splx[i].dee = splx[i].delta - splx[i].gamma*splx[i+1].dee;
		sply[i].dee = sply[i].delta - sply[i].gamma*sply[i+1].dee;
	}

	/* The coefficients are ai = vi and bi = Di (=dee[i]). Compute the remaining ci and di */

	for (i=0; i<n; i++)
	{
		splx[i].c = 3*(splx[i+1].v-splx[i].v) - 2*splx[i].dee - splx[i+1].dee;
		sply[i].c = 3*(sply[i+1].v-sply[i].v) - 2*sply[i].dee - sply[i+1].dee;
		splx[i].d = 2*(splx[i].v - splx[i+1].v) + splx[i].dee + splx[i+1].dee;
		sply[i].d = 2*(sply[i].v - sply[i+1].v) + sply[i].dee + sply[i+1].dee;
	}

	/* Finished. From now on, we can use splx and sply to interpolate between vertices */

	spline_prep = 1;

}


vertex splineintp (int vertexno, double t)
{
vertex v;
double r;

	v.x=0; v.y=0;
	if (!spline_prep) return v;

	r = splx[vertexno].v + (splx[vertexno].dee + (splx[vertexno].c + splx[vertexno].d*t)*t)*t;
	v.x = floor (0.4999+r);
	r = sply[vertexno].v + (sply[vertexno].dee + (sply[vertexno].c + sply[vertexno].d*t)*t)*t;
	v.y = floor (0.4999+r);
	return v;

}




/* Count the number of unique pixels along the trail */

int	tm_pixelcount (image_type *img)
{
int i,k;
rgbtriplet *p;
vertex v;
float t;


	p = img->data; k=0; pv.x=0; pv.y=0;
	if (!spline_prep) recompute_spline ();
	for (i=0; i<current_roi.numvertices-1; i++)
	{
		for (t=0; t<=1; t+=0.01)
		{
			v = splineintp (i,t);
			if ((v.x<0) || (v.y<0) || (v.x>=img->xmax) || (v.y >= img->ymax))
				dp (1, "Trailmaker - Fatal: Illegal interpolated values at v %d, t %5.2f: (%d,%d)\n",i,t,v.x,v.y);
			else
			{
				if ( (pv.x!=v.x) || (pv.y!=v.y) ) k++;
				pv = v;
			}
		}
	}
	dp (1, "Trailmaker: Trail needs %d vertices\n",k);
	return k;
}


/* Run along the spline and mark the trail in red
	--- dead code */

float follow_spline (image_type *img, int width)
{
int i,k,x,y;
double t,len;
rgbtriplet *p;
vertex v;
float *xdata,*ydata;
float buf;
char s[256];


	xdata = calloc (current_roi.numvertices*110, sizeof (float));
	ydata = calloc (current_roi.numvertices*110, sizeof (float));

	p = img->data; k=0; len=0.0; pv=splineintp (0,0);
	if (!spline_prep) recompute_spline ();
	for (i=0; i<current_roi.numvertices-1; i++)
	{
		for (t=0; t<=1; t+=0.01)
		{
			v = splineintp (i,t);
			if ((v.x<0) || (v.y<0) || (v.x>=img->xmax) || (v.y >= img->ymax))
				dp (1, "Trailmaker - Fatal: Illegal interpolated values at v %d, t %5.2f: (%d,%d)\n",i,t,v.x,v.y);
			else
			{
				buf = p[v.x+img->xmax*v.y].blue;
				ydata[k] = buf;
				xdata[k]=k;
				dp (2, "Trailmaker: Vertex %d, t=%4.2f: (%d,%d) h=%5.1f\n",i,t,v.x,v.y,buf);
				len += sqrt(ARITHM(pv.x-v.x, pv.y-v.y));
				k++;

				for (y=v.y-width; y<=v.y+width; y++)
					for (x=v.x-width; x<=v.x+width; x++)
						p[x+img->xmax*y].red=255;
				pv = v;
			}
		}
	}
	make_minmax (img);
	graphme (xdata,ydata,k);
	free (xdata); free (ydata);
	sprintf (s, "Trail length: %6.1f pixels, %7.2f km\n",len,len*5.6/1000.0);
	dp (1,s);
	return len;

}



/* Run along the spline and store each unique coordinate (i.e. not fractional
   pixels, but only integer pixels) and its height data.
   This function allocates memory which should be dealloctaed upon exit from
   the DSO.
   We pass an estimated number of points through totalpoints, and the
   function returns the actual number of pixels.
   This is a two-pass process. First, we get the integer discretization of
   the trail, then we remove all 90 degree pixels. */


int populate_data (image_type *img, int totalpoints)
{
int i,j,k;
double t;
rgbtriplet *p;
vertex v, pv, nv;
int *xtemp, *ytemp;
unsigned char b;


	xdata = (int*) calloc (totalpoints+200, sizeof (int));
	ydata = (int*) calloc (totalpoints+200, sizeof (int));
	hdata = (int*) calloc (totalpoints+200, sizeof (int));
	xtemp = (int*) calloc (totalpoints+200, sizeof (int));
	ytemp = (int*) calloc (totalpoints+200, sizeof (int));

	p = img->data; j=0; k=0; 
	if (!spline_prep) recompute_spline ();

	/* Initialize first vertex */

	pv=v=splineintp (0,0);
	xtemp[j]=v.x; ytemp[j]=v.y; j++;

	for (i=0; i<current_roi.numvertices-1; i++)		/* Each ROI vertex is a vertex of the spline */
	{
		for (t=0; t<=1; t+=0.01)					/* From one vertex to the next, t runs from 0 to 1 */
		{
			v = splineintp (i,t);
			if ((v.x<0) || (v.y<0) || (v.x>=img->xmax) || (v.y >= img->ymax))
				dp (1, "Trailmaker - Fatal: Illegal interpolated values at v %d, t %5.2f: (%d,%d)\n",i,t,v.x,v.y);
			else
			{
				if ( (pv.x!=v.x) || (pv.y!=v.y) )		/* We have advanced to the next vertex */
				{
					xtemp[j] = v.x;
					ytemp[j] = v.y;
					j++;
					pv = v;
				}

			}
		}
	}

	/* Pass 1 finished. Now test for right-angle pixels and delete them */

	pv.x=xtemp[j-1]; pv.y=ytemp[j-1];		/* Last point of trail */
	xtemp[j]=xtemp[0]; ytemp[j]=ytemp[0];	/* We have the space for +j rollover */

	for (i=0; i<j; i++)
	{
		b=0;
		v.x=xtemp[i]; v.y=ytemp[i];
		nv.x=xtemp[i+1]; nv.y=ytemp[i+1];
		if (pv.x!=v.x) b |= 1;
		if (pv.y!=v.y) b |= 2;
		if (nv.x!=v.x) b |= 4;
		if (nv.y!=v.y) b |= 8;
		if ((b!=6) && (b!=9))			/* Bit patterns that indicate a right angle */
		{
			xdata[k]=xtemp[i];
			ydata[k]=ytemp[i];
			hdata[k] = p[v.x+img->xmax*v.y].blue;
			k++;
			pv=v;
		}

	}

	return k;

}




/* Run along the trail pixels and mark the trail in red.
   Return length of trail. */

float follow_trail (image_type *img, int numpoints, int width)
{
int i,k,x,y,x1,y1;
float len;
rgbtriplet *p;

	if ((numpoints==0) || (xdata==NULL) || (ydata==NULL) || (hdata==NULL)) return 0.0;

	p = img->data; k=0; len=0.0;

	/* Restore r=g=b image from b data */

	for (i=0; i<img->xmax*img->ymax; i++) p[i].red = p[i].green = p[i].blue;

	for (i=0; i<numpoints; i++)
	{
			x = xdata[i]; y = ydata[i];
			if (i>0) len += sqrt(ARITHM( x-xdata[i-1], y-ydata[i-1] ));
			k++;
			for (y1=y-width; y1<=y+width; y1++)
				for (x1=x-width; x1<=x+width; x1++)
					p[x1+img->xmax*y1].red=255;
			p[x+img->xmax*y].green=255;

	}
	make_minmax (img);
	dp (1, "Trail length: %6.1f pixels, %7.2f km\n",len,len*5.6/1000.0);
	return len;

}



/* Draw height as a function of the trail position */

void draw_heightprofile (image_type *img, int numpoints)
{
int i;
float *xdt,*ydt;


	if ((numpoints==0) || (xdata==NULL) || (ydata==NULL) || (hdata==NULL)) return;

	xdt = calloc (numpoints, sizeof (float));
	ydt = calloc (numpoints, sizeof (float));
	for (i=0; i<numpoints; i++)
	{
		xdt[i] = i;
		ydt[i] = hdata[i];
	}
	graphme (xdt,ydt,numpoints);
	free (xdt);
	free (ydt);
}
	


/* Grading the trail: Make this an at least 3 pixel wide (~16 meter wide)
	flat trail. Flat means normal to direction of trail. 
	Run a filter over the height data and smooth the roughest slopes
	parallel to the trail direction. Filter goes up to 15.
*/

void grade_trail (image_type *img, int numpoints, int filter, int width)
{
int i,j,k,x,y,x1,y1,nx,ny;
float len, buf;
rgbtriplet *p;
int prev_h,h;
float *xdt,*ydt;

	if ((numpoints==0) || (xdata==NULL) || (ydata==NULL) || (hdata==NULL)) return;

	p = img->data; k=0; len=0.0;
	xdt = calloc (numpoints, sizeof (float));
	ydt = calloc (numpoints*2+20, sizeof (float));

	/* Restore r=g=b image from b data */

	for (i=0; i<img->xmax*img->ymax; i++) p[i].red = p[i].green = p[i].blue;

	/* Flatten a parking lot at trail start */

	x = xdata[0]; y = ydata[0]; h = p[x+img->xmax*y].green;
	for (y1=y-width-1; y1<=y+width+1; y1++)
		for (x1=x-width-1; x1<=x+width+1; x1++)
			p[x1+img->xmax*y1] = (rgbtriplet){h,h,h};
	hdata[0]=h;

	for (i=0; i<numpoints-1; i++)
	{
		x = xdata[i]; y = ydata[i]; /* h = p[x+img->xmax*y].green; */
		h = hdata[i];	/* We must not rely on image data as it may have changed in the flattening process */
		ydt[i]=h;

		if (i>0)
		{
			buf = ( (float)filter*(float)prev_h + (20.0-filter)*h )/20.0;		/* IIR filter */
			h = (int)(0.499+buf);
		}

		/* Flatten normal to the trail. We have eight possible directions: */

		if (i>0)
		{
			nx= xdata[i-1]; ny=ydata[i-1];			/* One point back */
			if ((nx<x) && (ny==y))				/* one step in X direction: East */
			{
				for (y1=y-width; y1<=y+width; y1++)				/* Grade north to south */
					p[x+img->xmax*y1] = (rgbtriplet){h,h,h};
			}
			else if ((nx<x) && (ny>y))			/* Northeast */
			{
				for (j=-width; j<=width; j++)					/* Grade northwest to southeast */
					p[(x+j)+img->xmax*(y+j)] = (rgbtriplet){h,h,h};
			}
			else if ((nx==x) && (ny>y))					/* one step in -Y direction: North */
			{
				for (x1=x-width; x1<=x+width; x1++)				/* Grade West to East */
					p[x1+img->xmax*y] = (rgbtriplet){h,h,h};
			}
			else if ((nx>x) && (ny>y))			/* Northwest */
			{
				for (j=-width; j<=width; j++)					/* Grade northeast to southwest */
					p[(x+j)+img->xmax*(y-j)] = (rgbtriplet){h,h,h};
			}
			else if ((nx>x) && (ny==y))				/* one step in -X direction: West */
			{
				for (y1=y-width; y1<=y+width; y1++)				/* Grade north to south */
					p[x+img->xmax*y1] = (rgbtriplet){h,h,h};
			}
			else if ((nx>x) && (ny<y))			/* Southwest */
			{
				for (j=-width; j<=width; j++)					/* Grade northwest to southeast */
					p[(x+j)+img->xmax*(y+j)] = (rgbtriplet){h,h,h};
			}
			else if ((nx==x) && (ny<y))					/* one step in Y direction: South */
			{
				for (x1=x-width; x1<=x+width; x1++)				/* Grade West to East */
					p[x1+img->xmax*y] = (rgbtriplet){h,h,h};
			}
			else if ((nx<x) && (ny<y))			/* Southeast */
			{
				for (j=-width; j<=width; j++)					/* Grade northeast to southwest */
					p[(x+j)+img->xmax*(y-j)] = (rgbtriplet){h,h,h};
			}
		}

		nx= xdata[i+1]; ny=ydata[i+1];			/* One point ahead */
		if ((x<nx) && (y==ny))				/* one step in X direction: East */
		{
			for (y1=y-width; y1<=y+width; y1++)				/* Grade north to south */
				p[x+img->xmax*y1] = (rgbtriplet){h,h,h};
		}
		else if ((x<nx) && (y>ny))			/* Northeast */
		{
			for (j=-width; j<=width; j++)					/* Grade northwest to southeast */
				p[(x+j)+img->xmax*(y+j)] = (rgbtriplet){h,h,h};
		}
		else if ((x==nx) && (y>ny))					/* one step in -Y direction: North */
		{
			for (x1=x-width; x1<=x+width; x1++)				/* Grade West to East */
				p[x1+img->xmax*y] = (rgbtriplet){h,h,h};
		}
		else if ((x>nx) && (y>ny))			/* Northwest */
		{
			for (j=-width; j<=width; j++)					/* Grade northeast to southwest */
				p[(x+j)+img->xmax*(y-j)] = (rgbtriplet){h,h,h};
		}
		else if ((x>nx) && (y==ny))				/* one step in -X direction: West */
		{
			for (y1=y-width; y1<=y+width; y1++)				/* Grade north to south */
				p[x+img->xmax*y1] = (rgbtriplet){h,h,h};
		}
		else if ((x>nx) && (y<ny))			/* Southwest */
		{
			for (j=-width; j<=width; j++)					/* Grade northwest to southeast */
				p[(x+j)+img->xmax*(y+j)] = (rgbtriplet){h,h,h};
		}
		else if ((x==nx) && (y<ny))					/* one step in Y direction: South */
		{
			for (x1=x-width; x1<=x+width; x1++)				/* Grade West to East */
				p[x1+img->xmax*y] = (rgbtriplet){h,h,h};
		}
		else if ((x<nx) && (y<ny))			/* Southeast */
		{
			for (j=-width; j<=width; j++)					/* Grade northeast to southwest */
				p[(x+j)+img->xmax*(y-j)] = (rgbtriplet){h,h,h};
		}

		hdata[i] = h;
		xdt[i]=i;
		ydt[i+numpoints]=h;

		prev_h = h;
	}

	make_minmax (img);
	xdt[numpoints-1]=numpoints-1;
	ydt[numpoints-1]=ydt[numpoints-2]; ydt[2*numpoints-1]=ydt[2*numpoints-2];
	multi_graphme (xdt,ydt,numpoints,2);
	free (xdt);
	free (ydt);
}


/********************************************************************************/


GtkAdjustment *tm_gradient_adj, *tm_width_adj, *tm_filter_adj;
GtkWidget *tm_vertices_label, *tm_length_label;


void trailmaker_go ()
{
float len;
char s[256];
int k,w;

	if (spline_prep)
	{
		free (splx); free (sply);
		spline_prep=0; splx=NULL; sply=NULL;
	}

	w = tm_width_adj->value;
	k = tm_pixelcount (&mainimg);								/* Estimate number of pixels for trail */
	pixelcnt = populate_data (&mainimg, k);						/* Get integer trail (x,y) data */
	sprintf (s, "%d (%d)", current_roi.numvertices,pixelcnt);
	gtk_label_set_text (GTK_LABEL(tm_vertices_label), s);


	len=follow_trail (&mainimg,pixelcnt,w);					/* Draw trail in image and determine real-world length */
	sprintf (s, "%6.1f pixels, %7.2f km",len,len*5.6/1000.0);
	gtk_label_set_text (GTK_LABEL(tm_length_label), s);

	make_imlib_buffer_from_main_image (cur_slice);
	redraw (1);


}

void on_tm_recalcbutton_clicked (GtkButton *button, gpointer user_data)
{

	trailmaker_go ();

}


void on_tm_okbutton_clicked (GtkButton *button, gpointer user_data)
{
	grade_trail (&mainimg, pixelcnt, (int)tm_filter_adj->value, (int)tm_width_adj->value);
	make_imlib_buffer_from_main_image (cur_slice);
	redraw (1);
}


void on_tm_savebutton_clicked (GtkButton *button, gpointer user_data)
{
gchar* filename;
FILE* FP;
int i;

	if ((pixelcnt==0) || (xdata==NULL) || (ydata==NULL) || (hdata==NULL)) return;
	filename= ChooseFile ("Save data as (XYH comma delimited)");
	if (filename)
	{
		FP = fopen (filename, "rt");
		if (FP)	/* file exists */
		{
			fclose (FP);
			statusbar_display ("File exists. No data written.");
		}
		else
		{
			FP = fopen (filename, "wt");
			for (i=0; i<pixelcnt; i++)
				fprintf (FP, "%4d, %4d, %4d\n ", xdata[i], ydata[i], hdata[i]);

			fclose (FP);
			statusbar_display ("X-Y-H ASCII file written");
		}
	}

}


void on_tm_hbutton_clicked (GtkButton *button, gpointer user_data)
{
	draw_heightprofile (&mainimg, pixelcnt);
}


void on_tm_cancelbutton_clicked (GtkButton *button, gpointer user_data)
{

	if (spline_prep)
	{
		if (splx) { free (splx); splx=NULL; }
		if (sply) { free (sply); sply = NULL; }
		spline_prep=0;
	}
	if (pixelcnt > 0)
	{
		if (xdata) { free (xdata); xdata=NULL; }
		if (ydata) { free (ydata); ydata=NULL; }
		if (hdata) { free (hdata); hdata=NULL; }
		pixelcnt=0;
	}
	gtk_widget_destroy (GTK_WIDGET(user_data));

}






void create_tm_window (const gchar* title)
{
  GtkWidget *tm_window;
  GtkWidget *tm_table;
  GtkWidget *tm_hbox, *tm_vbox;
  GtkWidget *tm_label1;
  GtkWidget *tm_label2;
  GtkWidget *tm_label3;
  GtkWidget *tm_label4;
  GtkWidget *tm_label5;
  GtkWidget *tm_hseparator1;
  GtkWidget *tm_okbutton, *tm_cancelbutton, *tm_recalcbutton, *tm_hgraphbutton, *tm_savebutton;
  GtkWidget *tm_width_hscale;
  GtkWidget *tm_filter_hscale;
  GtkWidget *tm_gradient_hscale;
  GtkWidget *tm_hseparator2;
  char s[256];


	splx = NULL; sply = NULL; spline_prep=0;
	xdata = ydata = hdata = NULL; pixelcnt = 0;
	sprintf (s, "%d", current_roi.numvertices);
	tm_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (tm_window), "tm_window", tm_window);
	if (title==NULL)
		gtk_window_set_title (GTK_WINDOW (tm_window), "Trail Maker");
	else
		gtk_window_set_title (GTK_WINDOW (tm_window), title);

	tm_vbox = vbox_in_window (tm_window, NULL);
	tm_table= MakeTable (tm_window, tm_vbox, NULL, 2,6);

	tm_label1 = MakeLabel (tm_window,tm_table,"Trail width","tm_label1",0,0);
	tm_label2 = MakeLabel (tm_window,tm_table,"Filter strength","tm_label2",0,1);
	tm_label3 = MakeLabel (tm_window,tm_table,"Follow gradient","tm_label3",0,2);
	tm_label4 = MakeLabel (tm_window,tm_table,"Number of vertices","tm_label4",0,4);
	tm_label5 = MakeLabel (tm_window,tm_table,"Total trail length","tm_label5",0,5);

	
	tm_width_adj = GTK_ADJUSTMENT(gtk_adjustment_new (1, 1, 5, 1, 1, 0));
	tm_width_hscale = MakeHScale (tm_width_adj, tm_window, tm_table, "tm_width_hscale",1,0);
	gtk_scale_set_digits (GTK_SCALE (tm_width_hscale), 0);

	tm_filter_adj = GTK_ADJUSTMENT(gtk_adjustment_new (1, 1, 15, 1, 3, 0));
	tm_filter_hscale = MakeHScale (tm_filter_adj, tm_window, tm_table, "tm_filter_hscale",1,1);
	gtk_scale_set_digits (GTK_SCALE (tm_filter_hscale), 0);

	tm_gradient_adj = GTK_ADJUSTMENT(gtk_adjustment_new (0, 0, 16, 1, 4, 0));
	tm_gradient_hscale = MakeHScale (tm_gradient_adj, tm_window, tm_table, "tm_gradient_hscale",1,2);
	gtk_scale_set_digits (GTK_SCALE (tm_gradient_hscale), 0);

	tm_hseparator1 = MakeHsep (tm_window, tm_table, NULL, 0,3);
	tm_hseparator1 = MakeHsep (tm_window, tm_table, NULL, 1,3);
	
	tm_vertices_label = MakeLabel (tm_window,tm_table,s,"tm_label4a",1,4);
	tm_length_label = MakeLabel (tm_window,tm_table,"--- pixels (--- km)","tm_label5a",1,5);

	tm_hseparator2 = MakeHsep (tm_window, tm_table, NULL, 0,6);
	tm_hseparator2 = MakeHsep (tm_window, tm_table, NULL, 1,6);

	tm_hbox = hbox_in_window (tm_window, tm_vbox, NULL);

	tm_hgraphbutton = Button_in_table (tm_window, tm_hbox, "Show Height", "tm_hiehgtbutton", -1,-1, on_tm_hbutton_clicked);
	tm_recalcbutton = Button_in_table (tm_window, tm_hbox, "Recompute", "tm_recalctbutton", -1,-1, on_tm_recalcbutton_clicked);
	tm_savebutton = Button_in_table (tm_window, tm_hbox, "Save (x,y) data", "tm_savebutton", -1,-1, on_tm_savebutton_clicked);	
	tm_okbutton = Button_in_table (tm_window, tm_hbox, "Grade", "tm_okbutton", -1,-1, on_tm_okbutton_clicked);
	tm_cancelbutton = Button_in_table (tm_window, tm_hbox, "Dismiss", "tm_cancelbutton", -1,-1, NULL);

	gtk_signal_connect (GTK_OBJECT (tm_window), "destroy",
                    	GTK_SIGNAL_FUNC (on_tm_cancelbutton_clicked), tm_window);
	gtk_signal_connect (GTK_OBJECT (tm_cancelbutton), "clicked",
                    	GTK_SIGNAL_FUNC (on_tm_cancelbutton_clicked), tm_window);

	gtk_widget_show (tm_window);
}



void trailmaker_main ()
{

	undo_nexus (&mainimg);
	create_tm_window ("Trail Maker");


}







/**************************************************************

	This is the infrastructure we use to register with the 
	main program. The pointer "menufunction" points to the 
	main entry (the function to be called when the menu is 
	activated). Initmodule is the one function that the 
	main program calls (the name is fixed!) to initialize
	the module and to register the module with the menu.

***************************************************************/

void (*menufunction1)();


variable_type (*macrofunction)(unsigned char* s, int* i);


void module_trailmaker_fn1 ()
{

	if (file_ok) 
	{
		if (current_roi.roi_mode !=	ROI_POLY)
		{
			ShowMessage ("This function requires a trail\ndefined through a polygonal ROI");
		}
		else	if (mainimg.imgtype != B_RGB)
		{
			ShowMessage ("You need to convert the image to RGB first");
		}
		else
			trailmaker_main ();

	}
}





void initmodule ()
{


	dp (1, "Confocal extensions module: Initializer called\n");
	
	/* Register this module with the main menu */

	menufunction1 = &module_trailmaker_fn1;

	dso_menu_register ("Trailmaker", menufunction1);

	/* Register the module with the macro interpreter 
	
	macrofunction = &mac_track_particles;
	register_dso_macro ("TRACK_PARTICLES",10,MT_IMG,macrofunction);
	
	*/
	
}

