

/*
 *  functions.c - some functions used all over the place
 */


/*		CRYSTAL IMAGE 0.9.0 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */


#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <fftw3.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_rng.h>


#include "defs.h"
#include "functions.h"
#include "dlgbox.h"
#include "imgbuf.h"


/* Redefine fftw 2's fftw_real for fftw3: */

#ifndef fftw_real

#define fftw_real	double

#endif

/* In addition, the struct containing re and im no longer exists. Use
   these macros instead: */

#define c_re(C)	((C)[0])
#define c_im(C)	((C)[1])


/* We need a GSL random generator */

const gsl_rng_type *gsl_randgen_type;
gsl_rng *gsl_randgen;



/*****************************************************

	An important help for debugging. dp (debug-print)
	is equivalent to fprintf (stderr, ...), but
	only prints if the priority level pri of the message
	exceeds the currently selected debug level.
	If debuglevel == 0, no messages are printed.
*/



void dp (int pri, char *format,...)
{
va_list args;
char buf[1024];

	if (pri > debuglevel) return;
    va_start(args, format);
    vsnprintf(buf,1020,format,args);
    fprintf (stderr,"%s",buf);
}



/********************************************************

	Critical memory allocations lead to program exit

*********************************************************/


void fatal_memerr ()	/* Abort program when critical memory allocation failed */
{
	ShowMessageC ("Fatal error - no memory. Program exits","That sucks!");
	exit (1);
}


/********************************************************

	Small function to determine whether a string is empty 
   (null string or only blanks)

********************************************************/


char isempty (char* s)
{
int i;

	i=0;
	while ((i<strlen(s)) && ((s[i]==' ') || (s[i]=='\t') || (s[i]=='\n'))) i++;
	if (i<strlen(s)) return 0; else return 1;
}


/*************************************************************/

/* Initialize whatever we need for using GSL components
*/


void	init_gsl_fn ()
{

	/* Prepare a random generator */
	
	gsl_rng_env_setup();
	gsl_randgen_type = gsl_rng_default;
	gsl_randgen = gsl_rng_alloc (gsl_randgen_type);


	
	
}



/*************************************************************/


/* Function returns whether a slice is all-zero.
	Returns 1 if all components are zero or 0 if not */


char slice_is_zero (image_type* img, int slice)
{
char* p;
char b;
long i,l,e;

	l = img->xmax * img->ymax * DATALEN(img->imgtype);
	e = l*slice;
	b=0;
	p = img->data;
	
	for (i=e; i<e+l; i++) b |= p[i];
	
	if (b!=0) return 0; 
	
	dp (1, "Zero slice %d detected\n",slice);
	return 1;
}
	




/* Nifty integer square root lifted from NIH image
   (The algorithm used by this integer square root function is described
	in the document "square-root-code.txt" in the /pub/nih-image/documents
	directory on rsbweb.nih.gov.) */
	
int isqrt (long x)
{
long r, nr, m;

	r=0;
	m=0x40000000;
	do
	{
		nr = r + m;
		if (nr <= x)
		{
			x -= nr;
			r = nr + m;
		}
		r = r >> 1;
		m = m >> 2;
	}
	while (m);
	if (x > r) r++;
	return r;
}


/* Interpolation coefficients for cubic interpolation. We need 4 coefficients.
	Let X be a non-integer coordinate, then the interpolation uses the four
	nearest points as follows: k is the integer part of X and x is the
	fractional part of X. The points included in the support are
	p(k-1), p(k), p(k+1), and p(k+2).

	John Russ (p.219) suggests the convolution
		p(k-1)	*	(x+3)^3 - 4(x+2)^3 + 6(x+1)^3 - 4x^3
		p(k)	*	(x+2)^3 - 4(x+1)^3 + 6x^3
		p(k+1)	*	(x+1)^3 - 4x^3
		p(k+2)	*	x^3

	The cubic spline demands the following convolution:
		p(k-1)	*	( -a x^3 + 2a x^2 - a x)
		p(k)	*	( (2-a) x^3 + (a-3) x^2 + 1 )
		p(k+1)	*	( (a-2) x^3 + (3-2a) x^2 + a x )
		p(k+2)	*	( a x^3 - a x^2 )
	where a is a parameter. Often, a=-0.5 is chosen. With a=-0.75,	
	a Hamming-windowed sinc is optimally approximated.

	Choose one of the three. Note that the Russ formulation interpolates, but
	has a worse lowpass characteristic (i.e., blurring) than linear interpolation.
	On the other hand, the Russ formulation shows no ringing, whereas the cubic
	spline does have ringing-type over- and undershoots. The reason is that Russ'
	interpolation is based on a cubic B-spline (see e.g. Dan Olfe, pg. 176) which
	means that the interpolated surface DOES NOT go through the support points.
	Conversely, the cubic spline is also known as "parabolic blend" for a=-0.5, 
	see Dan Olfe pp 163-164 for a derivation. 

	Also note that this interpolation type may be used for ANY smooth
	(that is, twice differentiable) interpolation. Example: We want to create
	smooth curve between n vertices that have an X and Y component each.
	We may use the fractional component 0<x<1 and 0<y<1 of X and Y
	to compute a set of coefficients each and use these to move smoothly
	from vertex k to vertex k+1 (additionally using vertices k-1 and k+2
	for the cubic spline)

	The parameter "type" determines the type of interpolation.
	Use 0 for the cubic B-spline, 
		1 for the cubic spline with a=0.5,
		2 for the cubic spline with a=-0.75

*/
	


void cub_coeff (double *a, double x, unsigned char type)
{
double x2, x3;
double oneover6 = 0.16666666666666667;

	switch (type)
	{
		case 0:		/* Natural B-spline, Russ p. 219 - horrible for pixel interpolation */

			a[3] = CUB(x); a[2] = CUB(1.0+x); a[1] = CUB(2.0+x);
			a[0] = (CUB(3.0+x) - 4.0*a[1] + 6.0*a[2] - 4.0*a[3]) * oneover6;
			a[1] = (6.0*a[3] - 4.0*a[2] + a[1])*oneover6;
			a[2] = (a[2] - 4.0*a[3])*oneover6;
			a[3] *= oneover6;
			break;

		case 1:						/* Cubic spline, a=-0.5 */

			x2=x*x; x3=x2*x;
			a[0] = -0.5*(x3-x2-x2+x);
			a[1] = 1.5*x3 - 2.5*x2 + 1.0;
			a[2] = -1.5*x3 + x2+x2 + 0.5*x;
			a[3] = 0.5*(x3-x2);
			break;

		case 2: 					/* Cubic spline, a=-0.75, optimum sinc-Hamming approximation */

			x2=x*x; x3=x2*x;
			a[0] = -0.75*(x3-x2-x2+x);
			a[1] = 1.25*x3 - 2.25*x2 + 1.0;
			a[2] = -1.25*x3 + 1.5*x2 + 0.75*x;
			a[3] = 0.75*(x3-x2);
			break;

		default:
			a[0] = a[1] = a[2] = a[3] = 0;	/* Try to avoid this case :-) */
	}


}


/* WINDOWING function(s) - will be extended as needed. 
	For now, we only use Hamm, Hanning, and Welch windows.
	Note that x and y need to be normalized into the range +/- 1.
	Note also that all windows except Hamming go to almost zero at the edges,
	so we create one row/column of unrecoverable data.
	If the caller normalized x and y so that it does not reach +/- 1,
	then there won't be unrecoverable data. But this is the
	caller's responsibility. */


float windowfn (float buf, float x, float y, int wndtype)
{
float w;

	switch (wndtype)
	{
		case 0:							/* Welch */
			w = (1.0-SQR(x))*(1.0-SQR(y));
			break;
		case 1:							/* Bartlett/Parzen */
			w = (1.0-fabs(x))*(1.0-fabs(y));
			break;
		case 2:							/* Hann */
			w = 0.25*(1.0-cos(PI*(x+1.0)))*(1.0-cos(PI*(y+1.0)));
			break;
		case 3:							/* Hamming raised cosine window */
			w = (0.54-0.46*cos(PI*(x+1.0)))*(0.54-0.46*cos(PI*(y+1.0)));
			break;
		case 4: 						/* Gauss window with sigma=0.5 */
			w = exp(-x*x-y*y);
			break;
		default:
			w=1.0;
	}

	return buf*w;

}


/* SORTING: We need something like this all over the place.
	Implementation of the comb sort algorithm with gap adjustment
	We have two versions, one for float, one for int. 
	In both cases, we may pass a non-NULL array "links"
	which must contain integer elements, and the links
	array will be sorted synchronously with the main array
*/


int combsort_newgap (int oldgap)			/* Adjust gap size */
{
int gap; 
float shrink1 = 0.769231;		/* Reciprocal of 1.3 */
float shrink2 = 0.801711876;	/* reciprocal of 1/(1-exp(-phi)) or 1.247331 */

	gap = (int)(oldgap*shrink2);	/* Yes, we mean floor, not round */

	if( (gap == 9) || (gap == 10))
		gap = 11;					/* Gap adjustment */

	if(gap < 1) return 1;
	return gap;
}


void combsort_f (float *array, int *links, int size)
{
int gap,i,ibuf;
unsigned char swapped;
float buf;

	gap = size;

	do
	{
		swapped = 0;
		gap = combsort_newgap (gap);

		for (i=0; i< size-gap; i++)
		{
			if (array[i] > array[i+gap])
			{
				buf = array[i]; array[i] = array[i+gap]; array[i+gap] = buf;
				if (links) 		/* Swap this array, too, if non-NULL */
				{ 
					ibuf = links[i]; links[i] = links[i+gap]; links[i+gap] = ibuf; 
				}
				swapped = 1;
			}
		}

	}
	while ( (gap>1) || swapped);
}


void combsort_i (int *array, int *links, int size)
{
int gap,i,ibuf;
unsigned char swapped;
int buf, loopcnt;

	gap = size; loopcnt=0;
	dp (3, "Combsort: Array at start (size %d) is\n",size);
	for (i=0; i<size; i++) dp (3, "  %4d ",array[i]);
	dp (3, "\n");


	do
	{
		swapped = 0; loopcnt++;
		gap = combsort_newgap (gap);
		dp (3, "Combsort: New gap size is %d\n",gap);

		for (i=0; i< size-gap; i++)
		{
			if (array[i] > array[i+gap])
			{
				buf = array[i]; array[i] = array[i+gap]; array[i+gap] = buf;
				if (links) 		/* Swap this array, too, if non-NULL */
				{ 
					ibuf = links[i]; links[i] = links[i+gap]; links[i+gap] = ibuf; 
				}
				swapped = 1;
				dp (3, "Combsort: Swapping %d and %d\n",i,i+gap);
			}
		}

	}
	while ( ((gap>1) || swapped) && (loopcnt < SQR(size)*2));
	dp (3, "Combsort: Array at finish is\n");
	for (i=0; i<size; i++) dp (3, "  %4d ",array[i]);
	dp (3, "\n");

}

/*-------------------------------------------------------------------------*/

/* Perform a log-log fit into xdata/ydata from st to en.
 	Report the results in 
		H	= slope of fit
		intcpt = intercept
		r	= Pearson correlation
	
	The loglogfit function uses base-10 log.
*/


fitresult loglogfit (double* x, double* y, int st, int en)
{
double lgx,lgy,r,Sx,Sxx,Sy,Syy,Sxy,H,intcpt;
int i,j;
fitresult result;

	Sx=Sxx=Sy=Syy=Sxy=0; j=0;
	for (i=st; i<=en; i++)
	{
		lgx = SAFELOG10(x[i]); lgy = SAFELOG10(y[i]);
		Sx  += lgx;
		Sxx += SQR(lgx);
		Sy  += lgy;
		Syy += SQR(lgy);
		Sxy += lgx*lgy;
		j++;
	}
	H = (Sx*Sy - j*Sxy)/(Sx*Sx - j*Sxx);
	intcpt = (Sy-H*Sx)/j;
	r = (j*Sxy-Sx*Sy)/(sqrt(j*Sxx-Sx*Sx)*sqrt(j*Syy-Sy*Sy));

	result.H = H;
	result.intcpt=intcpt;
	result.r=r;
	return result;

}


/* Perform a linear LSQ fit into xdata/ydata from st to en.
 	Report the results in H, intcpt and r, 
*/

fitresult lsqfit (double* xdata, double* ydata, int st, int en)
{
double r,Sx,Sxx,Sy,Syy,Sxy,H,intcpt;
int i,j;
fitresult result;

	Sx=Sxx=Sy=Syy=Sxy=0; j=0;
	for (i=st; i<=en; i++)
	{
		Sx  += xdata[i];
		Sxx += SQR(xdata[i]);
		Sy  += ydata[i];
		Syy += SQR(ydata[i]);
		Sxy += xdata[i]*ydata[i];
		j++;
	}
	H = (Sx*Sy - j*Sxy)/(Sx*Sx - j*Sxx);
	intcpt = (Sy-H*Sx)/j;
	r = (j*Sxy-Sx*Sy)/(sqrt(j*Sxx-Sx*Sx)*sqrt(j*Syy-Sy*Sy));

	result.H = H;
	result.intcpt=intcpt;
	result.r=r;
	return result;

}



/*********************************************+

	R A N D O M   G E N E R A T O R S
	
***********************************************/



/* Random generator from numerical recipes:

Long period (> 2 � 1018) random number generator of L'Ecuyer with Bays-Durham
shuffle and added safeguards. Returns a uniform random deviate between 0.0 and
1.0 (exclusive of the endpoint values). Call with rndreg a negative integer to
initialize; thereafter, do not alter rndreg between successive deviates in a
sequence. 
RNMX should approximate the largest floating value that is less than 1.

*/


/* For the use of GSL random generators, see

http://www.gnu.org/software/gsl/manual/html_node/Sampling-from-a-random-number-generator.html

and for non-uniform random generators,

http://www.gnu.org/software/gsl/manual/html_node/The-Gaussian-Distribution.html

*/

/* Uniform random number between 0 (inclusive) and 1.0 (exclusive) */

float nr_rand2 () { return gsl_rng_uniform (gsl_randgen); }



/* 
	Returns a normally distributed deviate with zero mean and unit variance,
	using uniform deviates as source.
*/


float gauss_rnd_nogo () { return gsl_ran_ugaussian (gsl_randgen); }

float gauss_rnd ()
{

static int iset=0;
static float gset;
float fac,rsq,v1,v2;

	if (rndreg < 0) iset=0;				/* Reinitialize. */

	if (iset == 0) 						/* We don't have an extra deviate handy, so... */
	{
		do 
		{
			v1=2.0*nr_rand2 ()-1.0;		/* ...pick two uniform numbers in the square */
			v2=2.0*nr_rand2 ()-1.0;		/* extending from -1 to +1 in each direction, */
			rsq=v1*v1+v2*v2;				/* see if they are in the unit circle, */
		} 
		while (rsq >= 1.0 || rsq == 0.0);	/* and if they are not, try again. */

		fac=sqrt(-2.0*log(rsq)/rsq);

		 /* Now make the Box-Muller transformation to get two normal deviates. 
		 	Return one and save the other for next time. */

		gset=v1*fac;
		iset=1;
		return v2*fac;
	} 
	else								/* We have an extra deviate handy, */
	{
		iset=0;
		return gset;
	}

}




/*
	eran1:
	Returns an exponentially distributed, positive, random deviate 
	of unit mean (?), using nr_rand2() as the source of uniform deviates.

	Note about mean: If we use log10, mean is about 0.1, horrible dropoff.
		If we use log_e, mean is about 0.58, better dropoff.
		If needed, we can normalize by dividing by 0.58.

*/


float eran1 ()
{
float buf; 

	do 
		buf = nr_rand2(); 
	while (F_ISZERO(buf)); 
	return -log(buf);
}


/* pnoise, smooth_p_noise, intp_p_noise, perlin_noise:

	Noise generator for Perlin noise. Used in the generator module for Perlin landscapes.
	This Perlin-style noise generator produces 1/f Perlin noise with zero mean.
	We have four functions. The actual noise generator (pnoise), a deterministic
	pseudo-random generator which returns a pseudo-random number as a function of 
	an x,y vector, smooth_p_noise which runs a [1, 2, 1] kernel over the noise, and 
	intp_p_noise which uses a cosine interpolation to interpolate.
	Finally, perlin_noise synthesizes the Perlin noise value at x and y.
	Note that the spatial arguments of perlin_noise should be normalized to 0...1
	Credit for basic pseudocode: Hugo Elias, http://freespace.virgin.net/hugo.elias/
*/

/* Cosine interpolation (better than linear, not as good as cubic).
	x is the fraction, 0<=x<1; a and b are the numbers to interpolate between */

float cosine_interpolate (float a, float b, float x)
{
float fx;

	fx = (1.0 - cos (x*PI))*0.5;	/* Cosine-swing rising from 0 to 1 as x goes from 0 to 1 */
									/* guarantees that there is no slope discontinuity at vertices */
	return  a*(1.0-fx) + b*fx;
}


float pnoise (int x, int y, int level)		/* Box-distributed random numbers between -1 and 1 */
{
long n;
int  p0[16] = {	251, 257, 263, 269, 271, 277, 281, 283, 293, 307, 311, 313, 317, 331, 337, 347};	/* prime numbers */
long p2[16] = {	15667, 15671, 15679, 15683, 15727, 15731, 15733, 15737, 15739,
				15749, 15761, 15767, 15773, 15787, 15791, 15797};
long p3[16] = {	789181, 789221, 789227, 789251, 789311, 789323, 789331, 789343, 
				789367, 789377, 789389, 789391, 789407, 789419, 789443, 789473};
long p4[16] = {	1376312017, 1376312071, 1376312087, 1376312099, 1376312107, 1376312173, 1376312183, 1376312257, 
				1376312261, 1376312263, 1376312269,  1376312279, 1376312323, 1376312369, 1376312383, 1376312393};


	level = MIN(level,15);

	n = x + y*p0[level];
	n =  (n<<13) ^ n;
	n = (n * ( SQR(n) * p2[level] + p3[level]) + p4[level]) & 0x7fffffff;
	return 1.0 - (float)n/1073741824.0;

}


float smooth_p_noise (int x, int y, int level)
{
float smoothmask[9] = {0.0625, 0.125, 0.0625, 0.125, 0.25, 0.125, 0.0625, 0.125, 0.0625};
int x1,y1,i;
float buf;

	buf = 0.0; i=0;
	for (y1=y-1; y1<=y+1; y1++)
		for (x1=x-1; x1<=x+1; x1++)
			buf += pnoise (x1,y1,level)*smoothmask[i++];

	return buf;
}


float intp_p_noise (float xr, float yr, int level)
{
int x,y;
float fx, fy;
float v1, v2, v3, v4; 		/* Rnd numbers on integer vertices of a square containing xr, yr */
float buf1, buf2;

	x = (int)floor(xr);
	fx= xr-x;
	y = (int)floor(yr);
	fy= yr-y;


	v1 = pnoise (x, y, level);			/* We could use smooth_p_noise here, but it doesn't do very much */
	v2 = pnoise (x+1, y, level);
	v3 = pnoise (x, y+1, level);
	v4 = pnoise (x+1, y+1, level);

	buf1 = cosine_interpolate (v1, v2, fx);				/* Interpolate in x direction */
	buf2 = cosine_interpolate (v3, v4, fx);				/* Interpolate in x direction */
	return cosine_interpolate (buf1, buf2, fy);			/* Interpolate in y direction */

}

/* The actual Perlin noise synthesizer */

float perlin_noise (float x, float y, float H, int octaves)		/* H is the persistence, 0<H<1 */
{
int i;
float freq, amplitude, buf;

	amplitude = 1.0;
	freq = 2.0;			/* This number determines the lowest-level smoothness */
	buf = 0;

	for (i=0; i<octaves; i++)
	{
		buf += intp_p_noise (x*freq, y*freq, i)*amplitude;
		amplitude *= H;			/* Amplitude is power (H,i) */
		freq *= 2;				/* Frequency is power (2,i) */
	}

	return buf;
}





/* Fractal Brownian Motion random generator.
 	A time series representing fractional Brownian motion (fBm)  
 	is generated by the method of successive random additions 
 	It is differenced to generate a series representing fractional 
	Brownian noise (fBn) if requested.
	The algorithm was derived from the FORTRAN-program "sra"
	by G. Raymond and J.B. Bassingthwaighthe, U Washington.
	Input parameters are sd (initial standard deviation) and 
	h (Hurst coefficient).
	This function will always generate time series in powers of 2,
	so we'll generate more than needed and simply cut it off.
*/


void create_fbm (float* timeseries, float sd, float h, int numpoints, char make_fbm)
{
int nlog,n,id,idd,level,i;
long idum;
float* delta; float* fbm;


	g_assert ( (h>=0) && (h<=1) );
	g_assert ( (sd>0) );

	nlog=1; idum=-1;
	while (pow(2,nlog)<numpoints) nlog++;
	n = pow (2,nlog);
	dp (2, "FBM: %d data points requested, next power of 2 is 2^%d, %d points\n",numpoints, nlog,n);
 
	delta = (float*) calloc (nlog+1, sizeof (float));
	fbm   = (float*) calloc (n   +1, sizeof (float));
 
	for (level=0; level<nlog; level++)
		delta[level] = sd*pow(0.5,level*h)*sqrt(0.5)*sqrt(1 - pow(2,2*h-2));
		
	fbm[0]=0;		
	fbm[n-1] = sd*gauss_rnd();		/* FORTRAN: fbm(n) = sd * randn(3,1.0) */

	idd=n; id = n/2;
	for (level=0; level<nlog; level++)
	{
		for (i=id; i<=n-id; i+=idd)
			fbm[i]  =0.5*(fbm[i-id] + fbm[i+id]);
			
		for (i=0; i<=n; i+=id)
			fbm[i] = fbm[i] + delta[level] * gauss_rnd();
  
		idd/=2;
		id/=2;
	}

	/* Now generate the dataset. Memory alloc etc is duty of 
	the calling program */
	
	for (i=0; i<numpoints; i++)
	{
		if (make_fbm)
			timeseries[i]= fbm[i+1];		/* Fractal Brownian Motion */
		else
			timeseries[i]= fbm[i+1]-fbm[i];	/* Fractional Brownian Noise */
	}

	free (delta);
	free (fbm);

}


/* Fractional Brownian Motion random generator using spectral synthesis -
	 Function computes a series from specified amplitudes and
	 phases using a back Fourier transform.
	
	 The phases are randomly chosen from between plus/minus pi.
	 The amplitudes are the product of a random number drawn from a
	 Gaussian distribution and a power function (i**(-beta/2)).
	 Treating all the amplitudes as positive numbers, the random
	 part has a mean of approximately 0.80 with standard deviation 0.60.

	The algorithm is derived from the FORTRAN-program "ssm" by
	G. Raymond and J.B. Bassingthwaighthe, U Washington
	
	According to the above reference, the dataset should be
	8 times longer than requested, and only the first n/8 points
	be used (http://nsr.bioeng.washington.edu/Software/index.html)
*/

/* TODO: This is incomplete and must be finished with an IFFT */

#define PHASER	1.813799364	/* PI/sqrt(3) */


void create_fbm_spectral (float *timeseries, float sd, float beta, int numpoints, char make_fbm)
{
int nlog, n,i,j;
long idum;
double amp, phase;
double* fbm;
double* fftdata;

	ShowMessage ("FBM/FBN spectral synthesis is not yet fully implemented");

	nlog=1; idum=-1;
	while (pow(2,nlog)<numpoints) nlog++;
	n = pow (2,nlog);
	if (make_fbm)
		dp (1, "FBM ");
	else
		dp (1, "FBN ");
	dp (1, "(spectral): %d data points requested, next power of 2 is 2^%d, %d points\n",numpoints, nlog,n);
	fbm   = (double*) calloc (n   +1, sizeof (double));
	fftdata= (double*) calloc (4*n+1, sizeof (double));
 
	j=2;	/* We leave the D.C. component zero */
	for (i=0; i<n; i++)
	{
		amp = gauss_rnd () / pow(i+1,beta);	/* Decay amplitude with frequency, use Gaussian rnd */
		phase = 2*PI*(nr_rand2()-0.5);	/* Phase is from equally distributed noise */
		fftdata[j  ] = amp*cos(phase);	/* Real part of freq */
		fftdata[j+1] = amp*sin(phase);	/* Imaginary part of freq */
		fftdata[4*n-j] = amp*cos(phase);	/* Real part of negative freq */
		fftdata[4*n-j+1] = amp*sin(phase);	/* Imaginary part of negative freq */

		fftdata[j]=fftdata[j+1]=amp;
		j+=2;
	}
	
	/*	dft_core (fftdata, n, -1);			/* Inverse FFT */

	/* Post-adjustment - the inverse transformation returns nn*data */
/*	for (i=0; i<n; i++) fftdata[i]/=n;

	/* Now generate the dataset. Memory alloc etc is duty of 
	the calling program.
	The fft array returns data as real - imag - real - ...
	and we simply drop the imaginary parts. */
	
	j=0;
	for (i=0; i<numpoints; i++)
	{
		timeseries[i]= fftdata[j];
		if ((!make_fbm) && (i>0))
			timeseries[i]-=fftdata[j-2];
		j+=2;
	}
	
	free (fftdata);
	free (fbm);
}


/* 	2D Fractional Brownian Motion random generator using spectral synthesis -
	 Function computes a 2D image from specified amplitudes and
	 phases using a back Fourier transform in 2D
	 img should be an empty, unallocated placeholder.
*/


void create_2d_fbm_spectral (image_type *img, float beta, int numpoints)
{
int n,x,y;
float amp, phase, re, im, r;
image_type fftimg; float *p1;
long cidx_re, cidx_im;


	n = numpoints/2; cidx_re = n + n*numpoints; cidx_im = cidx_re+SQR(numpoints);
	if (allocate_image (&fftimg, B_FLOAT, numpoints,numpoints,2)==-1)
		fatal_memerr();
	p1 = fftimg.data;
 
	for (y=1; y<n; y++)			/* We start with 1 to leave the D.C. component zero */
	{
		for (x=1; x<n; x++)
		{
		
			/* Choose a random number by magnitude and phase. The random number
				amplitude decays with the frequency. Then convert into re-im. 
				If we want to keep the IFFT's imaginary part zero, the frequency
				data's imaginary part must be zero, as well. */
			
			r = sqrt(ARITHM(x,y));
			amp = gauss_rnd () / pow(r,beta);	/* Decay amplitude with frequency, use Gaussian rnd */
			phase = 2*PI*(nr_rand2()-0.5);	/* Phase is from equally distributed noise */
			re = amp*cos(phase);	/* Real part of freq */
			im = amp*sin(phase);	/* Imaginary part of freq */
			
			/* Stuff the data into the four quadrants of the FFT image */
	
			p1[cidx_re + x + y*numpoints] = re;		/* Quadrant I */
			p1[cidx_im + x + y*numpoints] = 0;
			p1[cidx_re - x - y*numpoints] = re;		/* Quadrant III */
			p1[cidx_im - x - y*numpoints] = 0;
			
			r = sqrt(ARITHM(x,y));
			amp = gauss_rnd () / pow(r,beta);	/* Decay amplitude with frequency, use Gaussian rnd */
			phase = 2*PI*(nr_rand2()-0.5);	/* Phase is from equally distributed noise */
			re = amp*cos(phase);	/* Real part of freq */
			im = amp*sin(phase);	/* Imaginary part of freq */
	
			p1[cidx_re - x + y*numpoints] = re;		/* Quadrant II */
			p1[cidx_im - x + y*numpoints] = 0;
			p1[cidx_re + x - y*numpoints] = re;		/* Quadrant IV */
			p1[cidx_im + x - y*numpoints] = 0;

		}
	}
	
	/* Done with the generation, now backtransform using standard IFFT */
	
	ifft2d (&fftimg);
	*img = fftimg;

}






/*********************************************+

	Color model conversions
	
***********************************************/


/* Convert a rgb pixel to hsv */

hsvtriplet rgb_to_hsv (rgbtriplet rgb)
{
hsvtriplet hsv;
int h,s,v,m;
	
	if ((rgb.red==0) && (rgb.green==0) && (rgb.blue==0))
	{
		hsv = (hsvtriplet){0,0,0};
	}
	else
	{
	
		if ((rgb.red>=rgb.green) && (rgb.red>=rgb.blue))
		{
			v = rgb.red; m = (rgb.green<rgb.blue? rgb.green : rgb.blue);
			h = (int) (60.0*(rgb.green-rgb.blue) / (float)(v-m));
		}
		else if ((rgb.green>rgb.red) && (rgb.green>rgb.blue))
		{
			v = rgb.green; m = (rgb.red<rgb.blue? rgb.red : rgb.blue);
			h = (int) (120.0+60.0*(rgb.blue-rgb.red) / (float)(v-m));
		}
		else
		{
			v = rgb.blue; m = (rgb.red<rgb.green? rgb.red : rgb.green);
			h = (int) (240.0+60.0*(rgb.red-rgb.green) / (float)(v-m));
		}
		s = (int) (255.0 *(1.0 - (double) m / (double) v));
		hsv.hue= (h<0?  h+360 : h);
		hsv.sat=s & 0xff;
		hsv.val=v & 0xff;
	}
	return hsv;
}


rgbtriplet hsv_to_rgb (hsvtriplet hsv)
{
int sector;
float f, s;
int p,q,t;
rgbtriplet rgb;

	if( hsv.sat == 0 ) 	/* Sat==0 means achromatic (=grey) with r=b=g */
	{
		rgb.red = rgb.green = rgb.blue = hsv.val;
	}
	else
	{

		if (hsv.hue<0) hsv.hue+=360;
		sector = floor (hsv.hue / 60.0);	/* color sectors 0 through 5 */
		f = hsv.hue/60.0 - sector;			/* fractional part of h */
		s = (float)hsv.sat / 255.0;			/* Let sat go from 0 to 1 */
		p = (int)(0.5+hsv.val * ( 1 - s ));
		q = (int)(0.5+hsv.val * ( 1 - s * f ));
		t = (int)(0.5+hsv.val * ( 1 - s * ( 1 - f )));

		switch (sector)
		{
			case 0:
				rgb.red = hsv.val;
				rgb.green = t;
				rgb.blue = p;
				break;
			case 1:
				rgb.red = q;
				rgb.green = hsv.val;
				rgb.blue = p;
				break;
			case 2:
				rgb.red = p;
				rgb.green = hsv.val;
				rgb.blue = t;
				break;
			case 3:
				rgb.red = p;
				rgb.green = q;
				rgb.blue = hsv.val;
				break;
			case 4:
				rgb.red = t;
				rgb.green = p;
				rgb.blue = hsv.val;
				break;
			case 5:
			default:
				rgb.red = hsv.val;
				rgb.green = p;
				rgb.blue = q;
				break;
		}
	}
	return rgb;
}


rgbtriplet rgb_to_ycrcb (rgbtriplet rgb)
{
float y, cr, cb;
int i;
rgbtriplet result;

		y  = 0.299*rgb.red + 0.587*rgb.green + 0.114*rgb.blue;
		cr = -0.169*rgb.red - 0.331*rgb.green + 0.5*rgb.blue;
		cb = 0.5*rgb.red - 0.419*rgb.green - 0.081*rgb.blue;

		/* Pack back into uchar. Note that Cr and Cb are packed as signed short with offset */

		i = IROUND(y); if (i>255) i=255; else if (i<0) i=0;
		result.red=i;		/* Integer Y component */
		i = IROUND(127.0+cr); if (i>255) i=255; else if (i<0) i=0;
		result.green=i;		/* Integer Cr component */
		i = IROUND(127.0+cb); if (i>255) i=255; else if (i<0) i=0;
		result.blue=i;		/* Integer Cb component */
		return result;
}







void imgminmax (image_type* img, float* mmin, float* mmax)
{
float min,max;

	if (img->imgtype==B_FLOAT)
	{
		min = img->minval.fmin;
		max = img->maxval.fmax;
	}
	else
	{
		min = img->minval.imin;
		max = img->maxval.imax;
	}
	*mmin = min; *mmax = max;
}






/******************************************************

	MATRIX FUNCTIONS and related stuff

******************************************************/


/* det2x2 returns the determinant of a 2x2 matrix, quick n simple.
	A is supposed to have four elements:
	A[0] is a11, A[1] is a12, A[2] is a21, A[3] is a22. */

MFLOAT det2x2 (MFLOAT *A)
{
	return (A[0]*A[3] - A[1]*A[2]);
}



/* Pythag (numerical Recipes) 
   Computes (a**2 + b**2 )**1/2 without destructive under- or overflow.
*/


MFLOAT pythag (MFLOAT a, MFLOAT b) 
{ 
MFLOAT absa, absb; 

	absa=fabs(a); absb=fabs(b); 
	if (absa > absb) 
		return absa*sqrt(1.0+SQR(absb/absa)); 
	else 
		return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}


void fswap (MFLOAT* a, MFLOAT* b)
{
MFLOAT c;

	c=*a; *a=*b; *b=c;
}


/* Multiply a n by n matrix A with a n-vector V. Upon exit, the
   result is in V */

void matmult (MFLOAT* A, MFLOAT* V, int n)
{
int i,j;
MFLOAT R[n];

	for (i=0; i<n; i++)
	{
		R[i]=0;
		for (j=0; j<n; j++) R[i]+=V[j]*A[j+n*i];
	}
	for (i=0; i<n; i++) V[i]=R[i];
}



/* Multiply a n by m matrix A with a n-vector V. Upon exit, the
   result is in R, a m-vector, allocated by the caller */

void matmulnm (MFLOAT *R, MFLOAT* A, MFLOAT* V, int n, int m)
{
int i,j;

	for (i=0; i<m; i++)
	{
		R[i]=0;
		for (j=0; j<n; j++) R[i]+=V[j]*A[j+n*i];
	}
}



/* Solve a linear equation system by Gauss-Jordan Elimination.
   This function accepts a matrix A (n by n elements, defined by
   index col + n*row) and a vector B (n elements). It returns
   the solution vector X in B and the inverse of A in A */


int matsolv (MFLOAT *A, MFLOAT *B, int n) 
{ 
int *colidx,*rowidx,*ipiv; 
int i,col,row,j,k,l,ll, pividx; 
MFLOAT max, buf, pivinv, temp; 

	/* Please note that we swap rows/cols in this function, effectively
	   working on the transposed matrix. The caller needs to prepare
	   the matrix the usual (for me usual) way, i.e. x-index is the
	   fast index, while this algorithm assumes the Y index as the fast
	   index. This is caused by the original way of handling a matrix
	   using an array of pointers, i.e. A[n][n]. */


	colidx = calloc (n, sizeof (int));
	rowidx = calloc (n, sizeof (int));
	ipiv = calloc (n, sizeof (int)); /* Sic, calloc. We need to init this with 0 */
	
	row=col=0; /* silence a compiler warning */
	
	for (i=0; i<n; i++) 
	{ 
		
		/* Look for a pivot element. It is the matrix' maximum that has not yet been pivoted */
		
		max=0.0;
		for (j=0; j<n; j++) 
		{
			if (ipiv[j] != 1) 
				for (k=0; k<n; k++) 
				{ 
					if (ipiv[k] == 0) 
					{ 
						if (fabs(A[n*j+k]) >= max) 
						{ 
							max = fabs(A[n*j+k]); 
							row=j; col=k; 
						} 
					} 
				} 
		}
		(ipiv[col])++; 
		
		/* row / col now indicate the location of the pivot element */
		
		if (row != col) 
		{ 
			for (l=0; l<n; l++) { temp=A[row*n+l]; A[row*n+l]=A[col*n+l]; A[col*n+l]=temp; }
			temp=B[row]; B[row]=B[col]; B[col]=temp;
		} 
		
		/* Divide the pivot row by the pivot element */
		
		rowidx[i]=row; 
		colidx[i]=col; 
		pividx = col + n*col;
		if (A[pividx] == 0.0) return -1; /* Singular matrix */
		pivinv=1.0/A[pividx]; A[pividx]=1.0; 
		for (l=0; l<n; l++) A[col*n+l] *= pivinv; 
		B[col] *= pivinv;
		for (ll=0; ll<n; ll++) 
			if (ll != col) 
			{ 
				buf=A[ll*n+col]; 
				A[ll*n+col]=0.0; 
				for (l=0; l<n; l++) A[ll*n+l] -= A[col*n+l]*buf; 
				B[ll] -= B[col]*buf; 
			} 
	}
	
	for (l=n-1; l>=0; l--) 
	{ 
		if (rowidx[l] != colidx[l]) 
			for (k=0; k<n; k++) 
			{
				temp = A[k*n+rowidx[l] ]; 
				A[k*n+rowidx[l] ] = A[k*n+colidx[l] ];
				A[k*n+colidx[l] ] = temp;
			}
	} 
				
	free (ipiv); 
	free (rowidx); 
	free (colidx); 
	return 0;

}





void mat_transpose (MFLOAT *A, int n)
{
int x,y;
MFLOAT temp;

	for (y=0; y<n; y++)
		for (x=y+1; x<n; x++)
			if (x!=y)
			{
				temp = A[ x+n*y ];
				A[ x+n*y ] = A[ y+n*x ];
				A[ y+n*x ] = temp;
			}

}


/***************************************************************

Householder reduction of a real, symmetric matrix A.
A is passed as linear n by n array, and we use the macro MA
to access the elements.
On output, A is replaced by the orthogonal matrix Q effecting the
transformation. 
D[0..n-1] returns the diagonal elements of the tridiagonal matrix,
and E[0..n-1] the the off-diagonal elements with e[0]=0;
*/

#define MA(X,Y)		A[(X)+n*(Y)]
/*#undef MFLOAT
#define MFLOAT float*/

void householder_reduction (MFLOAT* A, int n, MFLOAT* D, MFLOAT* E)
{ 
int i,j,k,l;
MFLOAT scale,hh,h,g,f; 


	for (i=n-1; i>=1; i--) 
	{ 
		l=i-1; 
		h=0.0; scale=0.0; 
		if (l > 0)
		{ 
			for (k=0; k<=l; k++) 
				scale += fabs(MA(i,k)); 

			if (scale == 0.0) 	/* Skip transformation. */
				E[i]=MA(i,l); 
			else 
			{ 
				for (k=0; k<=l; k++) 
				{ 
					MA(i,k) /= scale; 		/* Use scaled A's for transformation */
					h += MA(i,k)*MA(i,k); 	/* Form sigma in h */
				} 
				
				f=MA(i,l);
				g = (f >= 0.0 ? -sqrt(h) : sqrt(h)); 
				E[i]=scale*g; 
				h -= f*g; 
				MA(i,l)=f-g; 				/* Store u in the i-th row of a  */
				
				f=0.0; 
				for (j=0; j<=l; j++) 
				{ 
					MA(j,i)=MA(i,j)/h; 		/* Store u/H in i-th column of a  */

					g=0.0; 					/* Form a element of A � u in g */
					for (k=0; k<=j; k++)  g += MA(j,k)*MA(i,k);
					for (k=j+1; k<=l; k++)  g += MA(k,j)*MA(i,k);
					E[j]=g/h; 				/* Form element of p in temporarily unused element of e */
					f += E[j]*MA(i,j);
				}
				
				hh=f/(h+h);
				for (j=0; j<=l; j++) 
				{ 
					f=MA(i,j);
					g=E[j]-hh*f; 
					E[j] = g;
					for (k=0; k<=j; k++) MA(j,k) -= (f*E[k]+g*MA(i,k)); 
				} 
			} 
		}
		else 
			E[i]=MA(i,l);
		
		D[i]=h; 
	}
	
	D[0]=0.0; E[0]=0.0; 
	for (i=0; i<n; i++) 
	{ 				/* Begin accumulation of transformation matrices. */
		l=i-1; 
		if (D[i]) 
		{ 							/* This block skipped when i==0 */
			for (j=0; j<=l; j++)
			{
				g=0.0; 
				for (k=0; k<=l; k++) 	/* Use u and u/H stored in A to form P � Q */
					g += MA(i,k)*MA(k,j);
				for (k=0; k<=l; k++) 
					MA(k,j) -= g*MA(k,i);
			} 
		} 

		D[i]=MA(i,i);
		MA(i,i)=1.0; 	/* Reset row and colum of A to identity matrix for next iteration. */

		for (j=0; j<=l; j++) MA(j,i)=MA(i,j)=0.0; 

	}

}


/*
QL algorithm with implicit shifts, to determine the eigenvalues 
and eigenvectors of a real, symmetric, tridiagonal matrix, 
or of a real, symmetric matrix previously reduced by Householder reduction.
On input, D[0..n-1] contains the diagonal elements of the 
tridiagonal matrix. On output, it returns the eigenvalues. 
The vector E[0..n-1] inputs the subdiagonal elements of the 
tridiagonal matrix, with E[0] arbitrary. 
On output E is destroyed. 
If the eigenvectors of a tridiagonal matrix are desired, 
the matrix A (linear 0..n-1, 0..n-1) is input as the identity matrix. 
If the eigenvectors of a matrix that has been reduced by Householder
are required, then A is input as the matrix A output by Householder. 
In either case, the k-th column of A returns the 
normalized eigenvector corresponding to D[k].
NOTE THAT UPON OUTPUT A IS TRANSPOSED and sorted by D.
This is what we need for the K-L transform.
*/

#define XFRSIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))


void eigen_ql (MFLOAT* A, int n, MFLOAT* D, MFLOAT* E)
{
int m,l,iter,i,k; 
MFLOAT s,r,p,g,f,dd,c,b; 
MFLOAT max; int j, maxidx;

	for (i=1; i<n; i++) 
		E[i-1]=E[i]; 							/* Convenient to renumber the el-ements of e. */
	
	E[n-1]=0.0; 
	for (l=0; l<n; l++) 
	{ 
		iter=0; 
		do 
		{ 
			for (m=l; m<n-1; m++) 
			{ 									/* Look for a single small subdiagonal element to split the matrix. */
				dd=fabs(D[m])+fabs(D[m+1]); 
				if ((float)(fabs(E[m])+dd) == (float)dd)
				{
					dp (3, "Found small element at %d\n",m);
					break; 
				}
			} 
			if (m != l)
			{ 
				if (iter++ == 30) 
					dp (0,"Too many iterations in eigenvector iteration"); 

				g=(D[l+1]-D[l])/(2.0*E[l]); 		/* Form shift. */
				r=pythag(g,1.0); 
				g=D[m]-D[l]+E[l]/(g+XFRSIGN(r,g)); 	/* This is dm - ks. */
				s=c=1.0; p=0.0;

				for (i=m-1;i>=l;i--) 
				{ 								/* A plane rotation as in the original QL, followed by Givens rotations to restore tridiagonal form. */
					f=s*E[i]; 
					b=c*E[i]; 
					E[i+1]=(r=pythag(f,g)); 
					if (r == 0.0) 
					{ 							/* Recover from underflow. */
						D[i+1] -= p; 
						E[m]=0.0;
						dp (3, "UNDERFLOW!\n");
						break; /* break exits the for-loop */
					} 
					
					s=f/r; 
					c=g/r; 
					g=D[i+1]-p; 
					r=(D[i]-g)*s+2.0*c*b; 
					D[i+1]=g+(p=s*r); 
					g=c*r-b;
					for (k=0; k<n; k++) 	
					{ 							/* Form eigenvectors. */
						dp (3,"Before: f=A(%d,%d)=%f, A(%d,%d)=%f,\n",k,i+1,MA(k,i+1),k,i,MA(k,i));
						f=MA(k,i+1); 
						MA(k,i+1)=s*MA(k,i)+c*f; 
						MA(k,i)=c*MA(k,i)-s*f; 
						dp (3,"After : s=%f, c=%f, A(%d,%d)=%f, A(%d,%d)=%f,\n",s,c,k,i+1,MA(k,i+1),k,i,MA(k,i));
					} 
				} 
				dp (3,"***    r=%f\n",r);
				/*
				if (r == 0.0 && i >= l) continue; /* cont skips the rest and cycles the for-loop */
				if ((r!=0.0) || (i<l))
				{
					D[l] -= p; E[l]=g; E[m]=0.0; 
				}
			} 
		} 
		while (m != l); 
	}

	/* Finally, sort the rows of the transposed A by descending D */

	for (i=0; i<n-1; i++)
	{
		max=D[i]; maxidx=i;
		for (j=i+1; j<n; j++)
		{
			if (D[j] > max) { max=D[j]; maxidx=j; }
		}
		if (maxidx!=i)
		{
			r=D[i]; D[i]=D[maxidx]; D[maxidx]=r;
			for (j=0; j<n; j++)
			{
				r=MA(j,i); MA(j,i)=MA(j,maxidx); MA(j,maxidx)=r;
			}
		}
	}
}




				


/* To verify this in Octave use

	a=zeros(3,3);
	a(1,:)=[3,2,7];
	a(2,:)=[2,5,6];
	a(3,:)=[7,6,1];
	q=orth(a);
	[v,d]=eig(a);
*/


void testmatrix ()
{
int n=3;
/*MFLOAT A[9] = { 3,2,7,2,5,6,7,6,1};*/
/*MFLOAT A[9] = { 2,0,1,0,2,0,1,0,2};*/
MFLOAT A[9] = { 0.996094, 0.953125, 0.726563, 0.953125, 1.9375, 1.28125, 0.726563, 1.28125, 1.359375};
MFLOAT D[n], E[n];
int i,j;


	dp (0, "Symmetrical Matrix before:\n");

	for (j=0; j<n; j++)
	{
		for (i=0; i<n; i++)
		{
			dp (0, "%6.2f   ",MA(i,j));
		}
		dp (0,"\n");
	}

	householder_reduction (A,n,D,E);

	dp (0, "Orthogonal Matrix after reduction:\n");

	/* Should be 	0.5388    0.5266    0.6576
    				0.5859    0.3266   -0.7417
    				0.6053   -0.7849    0.1325 (matlab)
					
					0.65    -0.76     0.00
 					-0.76   -0.65     0.00
  					0.00     0.00     1.00	(n.r.)

	*/

	for (j=0; j<n; j++)
	{
		for (i=0; i<n; i++)
		{
			dp (0, "%6.2f   ",MA(i,j));
		}
		dp (0,"\n");
	}
	
	dp (0, "Diagonal elements D:     ");
	for (j=0; j<n; j++) dp (0, "%6.2f  ",D[j]);
	dp (0,"\nOff-diagonal elements E: ");
	for (j=0; j<n; j++) dp (0, "%6.2f  ",E[j]);
	dp (0,"\n");

	/* Eigenvalues are     2.1548, -6.193, 13.0384 */


	eigen_ql (A,n,D,E);
	
	dp (0, "Eigen Matrix after QL algorithm:\n");

	/* Should be 	   	0.6576   -0.5266    0.5388
   						-0.7417   -0.3266    0.5859
    					0.1325    0.7849    0.6053


	*/

	for (j=0; j<n; j++)
	{
		for (i=0; i<n; i++)
		{
			dp (0, "%6.2f   ",MA(i,j));
		}
		dp (0,"\n");
	}
	
	dp (0, "Eigenvalues D:     ");
	for (j=0; j<n; j++) dp (0, "%6.2f  ",D[j]);
	dp (0,"\n");

	/* Eigenvalues are     2.1548, -6.193, 13.0384 */

	


}



#undef MA


/***************************************************************

	Spline-based interpolation

 *	This section is based on the following three papers:
 *		[1]	M. Unser,
 *			"Splines: A Perfect Fit for Signal and Image Processing,"
 *			IEEE Signal Processing Magazine, vol. 16, no. 6, pp. 22-38,
 *			November 1999.
 *		[2]	M. Unser, A. Aldroubi and M. Eden,
 *			"B-Spline Signal Processing: Part I--Theory,"
 *			IEEE Transactions on Signal Processing, vol. 41, no. 2, pp. 821-832,
 *			February 1993.
 *		[3]	M. Unser, A. Aldroubi and M. Eden,
 *			"B-Spline Signal Processing: Part II--Efficient Design and Applications,"
 *			IEEE Transactions on Signal Processing, vol. 41, no. 2, pp. 834-848,
 *			February 1993.
 *
 *	Credits:  Philippe Thevenaz, EPFL/STI/IOA/BIG (Lausanne, CH)
 *
***************************************************************/

image_type splcoeff;


void compute_weights (double x, double y, int* xi, int* yi, double* xw, double* yw, int width, int height, int SplDeg)
{
int i,j,k;
int	Width2, Height2;
double	w, w2, w4, t, t0, t1;

	Width2 = 2 * width - 2;
	Height2 = 2 * height - 2;

	/* compute the interpolation indices */	
	
	if (SplDeg & 1) 	/* Degree 3 or 5 */
	{
		i = (int)floor(x) - SplDeg / 2;
		j = (int)floor(y) - SplDeg / 2;
		for (k = 0; k <= SplDeg; k++) 
		{
			xi[k] = i++;
			yi[k] = j++;
		}
	}
	else 	/* degree 2 or 4 */
	{
		i = (int)floor(x + 0.5) - SplDeg / 2;
		j = (int)floor(y + 0.5) - SplDeg / 2;
		for (k = 0; k <= SplDeg; k++) 
		{
			xi[k] = i++;
			yi[k] = j++;
		}
	}

	/* compute the interpolation weights */	
	
	switch (SplDeg) 
	{
		case 2:
			/* x */
			w = x - (double)xi[1];
			xw[1] = 3.0 / 4.0 - w * w;
			xw[2] = (1.0 / 2.0) * (w - xw[1] + 1.0);
			xw[0] = 1.0 - xw[1] - xw[2];
			/* y */
			w = y - (double)yi[1];
			yw[1] = 3.0 / 4.0 - w * w;
			yw[2] = (1.0 / 2.0) * (w - yw[1] + 1.0);
			yw[0] = 1.0 - yw[1] - yw[2];
			break;
		case 3:
			/* x */
			w = x - (double)xi[1];
			xw[3] = (1.0 / 6.0) * w * w * w;
			xw[0] = (1.0 / 6.0) + (1.0 / 2.0) * w * (w - 1.0) - xw[3];
			xw[2] = w + xw[0] - 2.0 * xw[3];
			xw[1] = 1.0 - xw[0] - xw[2] - xw[3];
			/* y */
			w = y - (double)yi[1];
			yw[3] = (1.0 / 6.0) * w * w * w;
			yw[0] = (1.0 / 6.0) + (1.0 / 2.0) * w * (w - 1.0) - yw[3];
			yw[2] = w + yw[0] - 2.0 * yw[3];
			yw[1] = 1.0 - yw[0] - yw[2] - yw[3];
			break;
		case 4:
			/* x */
			w = x - (double)xi[2];
			w2 = w * w;
			t = (1.0 / 6.0) * w2;
			xw[0] = 1.0 / 2.0 - w;
			xw[0] *= xw[0];
			xw[0] *= (1.0 / 24.0) * xw[0];
			t0 = w * (t - 11.0 / 24.0);
			t1 = 19.0 / 96.0 + w2 * (1.0 / 4.0 - t);
			xw[1] = t1 + t0;
			xw[3] = t1 - t0;
			xw[4] = xw[0] + t0 + (1.0 / 2.0) * w;
			xw[2] = 1.0 - xw[0] - xw[1] - xw[3] - xw[4];
			/* y */
			w = y - (double)yi[2];
			w2 = w * w;
			t = (1.0 / 6.0) * w2;
			yw[0] = 1.0 / 2.0 - w;
			yw[0] *= yw[0];
			yw[0] *= (1.0 / 24.0) * yw[0];
			t0 = w * (t - 11.0 / 24.0);
			t1 = 19.0 / 96.0 + w2 * (1.0 / 4.0 - t);
			yw[1] = t1 + t0;
			yw[3] = t1 - t0;
			yw[4] = yw[0] + t0 + (1.0 / 2.0) * w;
			yw[2] = 1.0 - yw[0] - yw[1] - yw[3] - yw[4];
			break;
		case 5:
			/* x */
			w = x - (double)xi[2];
			w2 = w * w;
			xw[5] = (1.0 / 120.0) * w * w2 * w2;
			w2 -= w;
			w4 = w2 * w2;
			w -= 1.0 / 2.0;
			t = w2 * (w2 - 3.0);
			xw[0] = (1.0 / 24.0) * (1.0 / 5.0 + w2 + w4) - xw[5];
			t0 = (1.0 / 24.0) * (w2 * (w2 - 5.0) + 46.0 / 5.0);
			t1 = (-1.0 / 12.0) * w * (t + 4.0);
			xw[2] = t0 + t1;
			xw[3] = t0 - t1;
			t0 = (1.0 / 16.0) * (9.0 / 5.0 - t);
			t1 = (1.0 / 24.0) * w * (w4 - w2 - 5.0);
			xw[1] = t0 + t1;
			xw[4] = t0 - t1;
			/* y */
			w = y - (double)yi[2];
			w2 = w * w;
			yw[5] = (1.0 / 120.0) * w * w2 * w2;
			w2 -= w;
			w4 = w2 * w2;
			w -= 1.0 / 2.0;
			t = w2 * (w2 - 3.0);
			yw[0] = (1.0 / 24.0) * (1.0 / 5.0 + w2 + w4) - yw[5];
			t0 = (1.0 / 24.0) * (w2 * (w2 - 5.0) + 46.0 / 5.0);
			t1 = (-1.0 / 12.0) * w * (t + 4.0);
			yw[2] = t0 + t1;
			yw[3] = t0 - t1;
			t0 = (1.0 / 16.0) * (9.0 / 5.0 - t);
			t1 = (1.0 / 24.0) * w * (w4 - w2 - 5.0);
			yw[1] = t0 + t1;
			yw[4] = t0 - t1;
			break;
		default:
			dp (0,"Invalid spline degree\n");
	}

	/* apply the mirror boundary conditions */	
	
	for (k = 0; k <= SplDeg; k++) 
	{
		xi[k] = (width == 1) ? (0) : ((xi[k] < 0) ?
			(-xi[k] - Width2 * ((-xi[k]) / Width2))
			: (xi[k] - Width2 * (xi[k] / Width2)));

		if (width <= xi[k]) 
		{
			xi[k] = Width2 - xi[k];
		}

		yi[k] = (height == 1) ? (0) : ((yi[k] < 0) ?
			(-yi[k] - Height2 * ((-yi[k]) / Height2))
			: (yi[k] - Height2 * (yi[k] / Height2)));

		if (height <= yi[k]) 
		{
			yi[k] = Height2 - yi[k];
		}
	}


}


/* Initialize the causal coefficients. Pass the sample values in c
   and the number of samples in ns. The actual pole is in z and the 
   admissible relative error in epsilon.
*/


double	init_causal (double* c, int ns, double z, double epsilon)
{
double	Sum, zn, z2n, iz;
long	n, Horizon;

	/* this initialization corresponds to mirror boundaries */
	Horizon = ns;
	if (epsilon > 0.0) 
	{
		Horizon = (long)ceil(log(epsilon) / log(fabs(z)));
	}
	
	if (Horizon < ns) 	/* accelerated loop */
	{
		zn = z;
		Sum = c[0];
		for (n = 1; n < Horizon; n++) 
		{
			Sum += zn * c[n];
			zn *= z;
		}
		return Sum;
	}
	else	/* full loop */ 
	{
		zn = z;
		iz = 1.0 / z;
		z2n = pow(z, (double)(ns - 1));
		Sum = c[0] + z2n * c[ns - 1];
		z2n *= z2n * iz;
		for (n = 1L; n <= ns - 2; n++) 
		{
			Sum += (zn + z2n) * c[n];
			zn *= z;
			z2n *= iz;
		}
		return Sum / (1.0 - zn * zn);
	}
}


/* Get initial anticausal coefficient. Parameters as above */

double	init_anticausal (double* c, int ns, double z)
{
	return ((z / (z * z - 1.0)) * (z * c[ns - 2] + c[ns - 1]));
}



/* Compute the interpolation coefficients. Input the sample values
   in c[]. C[] contains the coefficients on exit. Specify the number of
   samples (and coefficients) in ns, the poles in z[] and the 
   number of poles in np and the
   admissible relative error in epsilon.
*/


void get_interpol_coeff (double* c, int ns, double* z, int np, double epsilon)
{

double	Lambda = 1.0;
int	n, k;

	
	if (ns == 1) return; /* special case required by mirror boundaries */

	for (k = 0; k < np; k++) 	/* compute the overall gain */
	{
		Lambda = Lambda * (1.0 - z[k]) * (1.0 - 1.0 / z[k]);
	}

	for (n = 0; n < ns; n++) /* apply the gain */
	{
		c[n] *= Lambda;
	}
	
	for (k = 0L; k < np; k++) /* loop over all poles */
	{
		c[0] = init_causal (c, ns, z[k], epsilon);	/* causal initialization */
		
		for (n = 1; n < ns; n++) 	/* causal recursion */
		{
			c[n] += z[k] * c[n - 1];
		}
		
		c[ns - 1] = init_anticausal (c, ns, z[k]); 	/* anticausal initialization */
		
		for (n = ns - 2; n >= 0; n--) /* anticausal recursion */
		{
			c[n] = z[k] * (c[n + 1] - c[n]);
		}
	}
}


/* Compute the coefficients for one image plane (2D only). This is
   and in-place operation on img, where img must be pre-allocated
   and be of type B_FLOAT. The original image values must be copied
   into img->data. SplDeg is the spline degree.
*/


void compute_coeff (image_type* img, int SplDeg) 
{
double	*Line;
float* p;
double	Pole[2];
int np;
int x,y,idx;

	g_assert (img->imgtype == B_FLOAT);
	p = img->data;

	/* recover the poles from a lookup table. If an invalid spline degree
	   is given, pretend we wanted degree 2 */
	switch (SplDeg) 
	{
		case 2:
			np = 1; Pole[0] = sqrt(8.0) - 3.0;
			break;
		case 3:
			np = 1; Pole[0] = sqrt(3.0) - 2.0;
			break;
		case 4:
			np = 2;
			Pole[0] = sqrt(664.0 - sqrt(438976.0)) + sqrt(304.0) - 19.0;
			Pole[1] = sqrt(664.0 + sqrt(438976.0)) - sqrt(304.0) - 19.0;
			break;
		case 5:
			np = 2;
			Pole[0] = sqrt(135.0 / 2.0 - sqrt(17745.0 / 4.0)) + sqrt(105.0 / 4.0)
				- 13.0 / 2.0;
			Pole[1] = sqrt(135.0 / 2.0 + sqrt(17745.0 / 4.0)) - sqrt(105.0 / 4.0)
				- 13.0 / 2.0;
			break;
		default:
			np = 1; Pole[0] = sqrt(8.0) - 3.0;
			dp (0,  "Invalid spline degree selected\n");
	}

	/* Sample computation is an in-place separable process, so perform
	the operation twice; once along X then along Y. The tolerance epsilon
	is defined in math.h and is somewhere around 2.22e-16 */

	Line = calloc(img->xmax, sizeof(double));
	if (!Line) fatal_memerr();

	for (y = 0; y < img->ymax; y++) 
	{
		idx = img->xmax * y;
		for (x=0; x<img->xmax; x++) Line[x] = p[x+idx];
		get_interpol_coeff (Line, img->xmax, Pole, np, DBL_EPSILON);
		for (x=0; x<img->xmax; x++) p[x+idx] = Line[x];
	}
	free(Line);

	Line = calloc(img->ymax, sizeof(double));
	if (!Line) fatal_memerr();

	for (x = 0; x < img->xmax; x++) 
	{
		idx=0;
		for (y=0; y<img->ymax; y++) { Line[y] = p[x+idx]; idx+=img->xmax; }
		get_interpol_coeff (Line, img->ymax, Pole, np, DBL_EPSILON);
		idx=0;
		for (y=0; y<img->ymax; y++) { p[x+idx] = Line[y]; idx+=img->xmax; }
	}
	free(Line);
}


/* Here is the actual 2D interpolation function. It requires the coefficients
   to be pre-computed and cached in the "image" coeff. It does *NOT* require
   the original image data as this was already used to compute the coefficients.*/


double spline_interpol_core (image_type* coeff, float x, float y, int SplDeg)
{
float	*p;	
double	xWeight[6], yWeight[6];
double	interpolated;
double	w;
int		xIndex[6], yIndex[6];
int	i, j, idx;


	g_assert (coeff->imgtype == B_FLOAT);
	p = coeff->data;

	/* Indices and weights need to be computed for every x,y coordinate */

	compute_weights (x,y, xIndex, yIndex, xWeight, yWeight, coeff->xmax, coeff->ymax, SplDeg);

	/* perform interpolation */	
	
	interpolated = 0.0;
	for (j = 0; j <= SplDeg; j++) 
	{
		idx = yIndex[j]* coeff->xmax;
		w = 0.0;
		for (i = 0; i <= SplDeg; i++) 
		{
			w += xWeight[i] * p[xIndex[i] + idx];
		}
		interpolated += yWeight[j] * w;
	}

	return(interpolated);
}


/* The spline interpolation routine with coefficient caching mechanism */
/* TODO: Can't handle RGB */

double spline_interpol (image_type* img, float xf, float yf, int z, int SplDeg)
{
int x,y;
float* p;

	if (!spl_cache_invalidate)		/* Check if we need to invalidate */
	{
		if (spl_cache_z != z) spl_cache_invalidate=1;
		if (spl_cache_deg != SplDeg) spl_cache_invalidate=1;
		if (spl_cache_uid != img->uid) spl_cache_invalidate=1;
		
		if (spl_cache_invalidate)
		{
		 	freebuf (&splcoeff);
			dp (1, "Spline interpolation: Cache invalidated\n");
		}
	}

	if (spl_cache_invalidate)		/* Need to recompute the coefficients */
	{
		
		dp (1, "Spline interpolation: Recomputing coefficients\n");
		if (allocate_image (&splcoeff, B_FLOAT, img->xmax, img->ymax, 1)==-1)
			fatal_memerr();
		p = splcoeff.data;
		
		for (y=0; y<img->ymax; y++)
			for (x=0; x<img->xmax; x++)
				p[x + img->xmax*y] = readbuf_flt (img, x,y,z);
		
		compute_coeff (&splcoeff, SplDeg);
		
		spl_cache_invalidate=0;
		spl_cache_z = z;
		spl_cache_deg=SplDeg;
		spl_cache_uid = img->uid;
	}
	
	return spline_interpol_core (&splcoeff, xf,yf, SplDeg);

}







/******************************************************************/




