

/* vertanalysis.c -- Vertebral bone analysis with CT.

	This module assumes that a segmetation run with the spiderop has been done, and that
	the spiderop variables, "rayforvertices" and "numvertices" are set.
	
*/


/*		CRYSTAL IMAGE 0.9.0 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "vertanalysis.h"
#include "roi.h"
#include "dlgbox.h"
#include "imgbuf.h"
#include "application.h"
#include "secondwnd.h"
#include "filter.h"
#include "graphdsp.h"
#include "gtkhelpers.h"
#include "spiderop.h"
#include "functions.h"


image_type spider_mask;
char* report;		/* String to display results */
int searchpathlen;	/* Length (#pixels) to scan for local maximum, either side */
vertex c_apex1,c_apex2;		/* Apex (top, bottom) of cortical ridge */



// //int round (float x) { return (int)(0.5+x); }

int isign (int i) { if (i>0) return 1; else if (i<0) return -1; else return 0; }


/* Take the rays we obtained from the spider op and follow in both directions to
   find the closest local maximum. For the spider rays, the center of the spider image
   is (0,0)
*/

void go_local_max (int dist)
{
int i,j,x,y,x1,y1,index;
float maxval, buf;
vertex locmax;
int distsqr;

	distsqr = SQR(dist);

	for (i=0; i<numvertices; i++)
	{
		/* Default: Assume maximum is at vertex */
		
		locmax.x = x = rayforvertices[i].vtxXPixelCoord;
		locmax.y = y = rayforvertices[i].vtxYPixelCoord;
		maxval = readbuf_flt (&spider_img, x+centerglobal.x, y+centerglobal.y, 0);
		index = rayforvertices[i].vtxPixelIndex;
		
		/* Run along the ray trace */

		for (j=0; j<rayforvertices[i].numPixels; j++)
		{
			x1 = rayforvertices[i].XPixelCoords[j];
			y1 = rayforvertices[i].YPixelCoords[j];
			if (SQR(x-x1)+SQR(y-y1) < distsqr )	/* maintain min distance from vertex */
			{
				buf = readbuf_flt (&spider_img, x1+centerglobal.x, y1+centerglobal.y, 0);
				if (buf>maxval)
				{			
					maxval=buf;
					locmax.x = x1; locmax.y = y1;
					index=j;
				}
			}
		}
				
		/* Update our ray */
		
		rayforvertices[i].maxXPixelCoord = locmax.x;
		rayforvertices[i].maxYPixelCoord = locmax.y;
		rayforvertices[i].maxPixelValue = maxval;
		rayforvertices[i].maxPixelIndex = index;
	}
		
}


/**********************************************************************
	
	The following code determines the cortical ridge and segments the cortical shell.
	
	The mask code is
		0:	Nothing of interest
		1:	Spongiosa
		2:	Cortical shell
		3:	Cortical ridge
		
	setbox2 sets the mask in the 3x3 neighborhood of a box to 2 if the threshold is exceeded
	handlebox2 looks for the local maximum near cx,cy
	
	Boxwalk: Walk from vertex to vertex (using Bresenham), and look at a n x n box
	around the current point. Find the maximum in that box and set every pixel above 
   	0.63 % of the maximum to 2 - this is the cortical shell. 
	
*/


#define BUF(xx,yy)	readbuf_flt (&spider_img, (xx)+centerglobal.x, (yy)+centerglobal.y, 0)

void setmask (int x, int y, int v)
{
char* p;

	p = spider_mask.data;
	x +=centerglobal.x; if (x<0) x=0; else if (x>=spider_mask.xmax) x=spider_mask.xmax-1;
	y +=centerglobal.y; if (y<0) y=0; else if (y>=spider_mask.ymax) y=spider_mask.ymax-1;
	if (p[ x + spider_mask.xmax*y ]<3) p[ x + spider_mask.xmax*y ] = v;
}

/* Linemask draws a straight line into the mask */

void linemask (int x1, int y1, int x2, int y2, int v)
{
int dx, dy, ax, ay, sgx, sgy, d, x, y;

	dx=x2-x1; dy=y2-y1; ax=2*abs(dx); ay=2*abs(dy);
	sgx=isign(dx); sgy=isign(dy);
	x=x1; y=y1;
	if (ax>ay)        /* x dominant */
	{
		d = ay - dx;
		while (x!=x2)
		{
			if (d>=0) { y+=sgy; d-=ax; }
			x+=sgx; d+=ay;
			setmask (x,y,v);
		}
	}
	else              /* y dominant */
	{
		d = ax - dy;
		while (y!=y2)
		{
			if (d>=0) { x+=sgx; d-=ay; }
			y+=sgy; d+=ax;
			setmask (x,y,v);
		}
	}
}


/* Setbox2 sets all pixels above a threshold (setval) in a 3x3 neighborhood around x,y */


void setbox2 (int x, int y, float setval)
{
int x1, y1, x2, y2, i, j;

	x1=x-1; y1=y-1; x2=x+1; y2=y+2;
	for (j=y1; j<=y2; j++) 
   		for (i=x1; i<=x2; i++) 
			if (BUF(i,j)>=setval) setmask(i,j,2);
}


/*  Handlebox 2 is called for each pixel (cx,cy) of the connecting line 
	between vertices.
	Run from the center to the margin through (cx,cy). Within a specified 
	distance (boxsize), look for the maximum value. This is one 
	pixel of the cortical ridge */

void handlebox2 (int cx, int cy, int boxsize, float thresh, float ofs)
{
int x1, x2, y1, y2, dx, dy, ax, ay, sgx, sgy, d, x, y;
int mx, my;
float max, dist, sqboxsize, threshval;
int thickcnt;
int maxX, maxY;

	sqboxsize = SQR(boxsize);
	maxX = spider_img.xmax;
	maxY = spider_img.ymax;
	x1=0; y1=0; /* Starting point, relative to spider center, i.e. (0,0) */
	x2=cx+cx-x1; y2=cy+cy-y1;	/* End pt, twice the distance */
	dx=x2-x1; dy=y2-y1; ax=2*abs(dx); ay=2*abs(dy);
	sgx=isign(dx); sgy=isign(dy); mx=cx; my=cy;
	max = BUF(cx,cy);
   
	if (ax>ay)        /* x-dominant: run from center to margin */
	{
		d = ay - ax/2; x=x1; y=y1;
		while (x!=x2)  /* Pass 1: find max along that line */
		{
			dist=SQR(x-cx)+SQR(y-cy);
			/* if ((dist<=sqboxsize) && (max<BUF(x,y))) { max = BUF(x,y); mx=x; my=y; } */
			if ((dist<=SQR(searchpathlen)) && (max<BUF(x,y))) { max = BUF(x,y); mx=x; my=y; }
			if (d>=0) { y+=sgy; d-=ax; }
			x+=sgx; d+=ay;
		}
		d = ay - ax/2; x=x1; y=y1; thickcnt=0;
		while (x!=x2) 	/* pass 2: set box (3x3) around that line */
		{
			dist=SQR(x-cx)+SQR(y-cy); threshval = thresh*max + (1-thresh)*ofs;
			if (dist<=sqboxsize) 
			{ 
				setbox2 (x,y, threshval); 
				if (BUF(x,y)>=threshval) thickcnt++; 
			}
			if (d>=0) { y+=sgy; d-=ax; }
			x+=sgx; d+=ay;
		}
	}
	else              /* y dominant */
	{
		d = ax - ay/2; x=x1; y=y1;
		while (y!=y2)
		{
			dist=SQR(x-cx)+SQR(y-cy);
			/*if ((dist<=sqboxsize) && (max<BUF(x,y))) { max = BUF(x,y); mx=x; my=y; }*/
			if ((dist<=SQR(searchpathlen)) && (max<BUF(x,y))) { max = BUF(x,y); mx=x; my=y; }
			if (d>=0) { x+=sgx; d-=ay; }
			y+=sgy; d+=ax;
		}
		d = ax - ay/2; x=x1; y=y1; thickcnt=0;
		while (y!=y2)
		{
			dist=SQR(x-cx)+SQR(y-cy); threshval = thresh*max + (1-thresh)*ofs;
			if (dist<=sqboxsize) 
			{ 
				setbox2 (x,y, threshval); 
				if (BUF(x,y)>=threshval) thickcnt++; 
			}
			if (d>=0) { x+=sgx; d-=ay; }
			y+=sgy; d+=ax;
		}
	}
	ridge[ridgemax++] = (ridge_element){ mx,my,max };
}



void boxwalk (int boxsize, float thresh)
{
int i,x,y,x0,y0,x1,y1;
int dx,ax, dy, ay,sgx,sgy,d;
double a, v1, v2, ofs;
int len, cnt;


	for (i=0; i<numvertices; i++)
	{
		rayforvertices[i].maxXPixelCoord = rayforvertices[i].vtxXPixelCoord;
		rayforvertices[i].maxYPixelCoord = rayforvertices[i].vtxYPixelCoord;
	}

	for (i=0;i<numvertices; i++)
	{
		x0 = rayforvertices[i].maxXPixelCoord;
		y0 = rayforvertices[i].maxYPixelCoord;
		v1 = rayforvertices[i].vtxOutsideValue;
		if (i==numvertices-1)
		{
			x1 = rayforvertices[0].maxXPixelCoord;
			y1 = rayforvertices[0].maxYPixelCoord;
			v2 = rayforvertices[0].vtxOutsideValue;
		}
		else
		{
			x1 = rayforvertices[i+1].maxXPixelCoord;
			y1 = rayforvertices[i+1].maxYPixelCoord;
			v2 = rayforvertices[i+1].vtxOutsideValue;
		}
		len=0; cnt=0;

		dp (2, "Vertanalysis: Vertex %d, Running from %d,%d to %d,%d\n",
					i,x0,y0,x1,y1);

		/* Bresenham: Run from x0,y0 to x1,y1; call handlebox2 for each point */

		dx=x1-x0; dy=y1-y0; ax=2*abs(dx); ay=2*abs(dy);
		sgx=isign(dx); sgy=isign(dy); x=x0; y=y0;
		if (ax>ay)        /* x dominant */
		{
 			d = ay - ax/2; len = abs(dx);
  			while (x!=x1)
 			{
				a = (double)cnt / (double) len; ofs = a*v2 + (1.0-a)*v1;
				handlebox2 (x,y,boxsize,thresh, ofs);
				if (d>=0) { y+=sgy; d-=ax; }
				x+=sgx; d+=ay; cnt++;
			}
		}
		else              /* y dominant */
		{
			d = ax - ay/2; len = abs(dy);
			while (y!=y1)
			{
				a = (double)cnt / (double) len; ofs = a*v2 + (1.0-a)*v1;
				handlebox2 (x,y,boxsize,thresh, ofs);
				if (d>=0) { x+=sgx; d-=ay; }
				y+=sgy; d+=ax; cnt++;
			}
		}
		
	}
	
	/* Finally, put the ridge into the mask, draw closed lines.
	    By doing this, the floodfill cannot escape any more.
		While we are at this, we may also find the apex */

	c_apex1 = c_apex2 = (vertex){ridge[0].x,ridge[0].y};
	for (i=1; i<ridgemax; i++)
	{
		linemask (ridge[i-1].x,ridge[i-1].y, ridge[i].x,ridge[i].y, 3);
		if (ridge[i].y < c_apex1.y) c_apex1 = (vertex){ridge[i].x, ridge[i].y};
		if (ridge[i].y > c_apex2.y) c_apex2 = (vertex){ridge[i].x, ridge[i].y};
	}
	c_apex1.x += centerglobal.x;
	c_apex1.y += centerglobal.y;
	c_apex2.x += centerglobal.x;
	c_apex2.y += centerglobal.y;
}


/********************************************************************

	The following is the flood-fill algorithm to fill the area
	enclosed in the cortical shell. Set to "1" indicates spongiosa.
	
*/


void fillpoly_sub (int x, int y)
{
 int status, x0, y0, maxX, maxY;
 char* p;
 
	status = 1;
	maxX = spider_mask.xmax; maxY = spider_mask.ymax;
	p = spider_mask.data;
	if ((x>=maxX) || (x<0) || (y>=maxY) || (y<0)) status=0; /* invalid point => abort */
	
	if (status)
	{


		if (p[x+maxX*y]>0) status=0;  /* Any point already visited and set */

		/* Border points have a value > 1. IF we find any border pts in the
      	 	vicinity (x|y itself and 8 neighbors) it is considered a border */
	
		if (status)
			for (y0=y-1; y0<=y+1; y0++)
				for (x0=x-1; x0<=x+1; x0++)
				{
					if (p[x0 + maxX*y] > 1) status=0; /* Cortical shell or ridge */
				}


		if (status)
		{
			p[x + maxX*y] = 1; 
			fillpoly_sub (x, y-1);
			fillpoly_sub (x, y+1);
			fillpoly_sub (x-1, y);
			fillpoly_sub (x+1, y);
		}	
	}		
}

/* The driver for the above subroutine */

void fillpoly (int x_seed, int y_seed)
{

	fillpoly_sub (x_seed, y_seed);   /* inital call for recursion */
   
}





/*************************************************************

	Visualize the segmentation result.
	Use secondary image s_img, assumed to exist!

*/

void vert_visualize ()
{
int x,y, stride, xmax, ymax;
long idx;
unsigned char *p;
unsigned char *s_im_buf;

	xmax = gdk_pixbuf_get_width (s_img.pixbuf);
	ymax = gdk_pixbuf_get_height (s_img.pixbuf);
	stride = gdk_pixbuf_get_rowstride (s_img.pixbuf);
	s_im_buf = gdk_pixbuf_get_pixels (s_img.pixbuf);			/* Pointer to pixbuf's rgb data */

	p = spider_mask.data;
	for (x=0; x<spider_img.xmax; x++)
		for (y=0; y<spider_img.ymax; y++)
		{
			idx = 3*x + stride*y;
			switch (p[x + spider_mask.xmax*y])
			{
				case 0:		/* Out of ROI, color it blue */
					s_im_buf[idx]=0;
					s_im_buf[idx+1]=0;
					break;
				case 1:		/* Spongiosa is colored red */
					s_im_buf[idx+1]=0;
					s_im_buf[idx+2]=0;
					break;
				case 2:		/* Cortex is green */
					s_im_buf[idx]=0;
					s_im_buf[idx+2]=0;
					break;
				case 3:		/* Cortical ridge is a bright yellow line */
					s_im_buf[idx]=0xff;
					s_im_buf[idx+1]=0xff;
					s_im_buf[idx+2]=0;
					break;
			}
		}

	secondary_image_redraw ();

}


/* vertebrae_process calls the sequence of processing functions. It has to match between
   vertebrae_main and vertebrae_meas, so it is extracted as a separate function */

void vertebrae_proc (int searchlen, int fboxsize, float fThresh)
{
int x,y;
char* p = spider_mask.data;

	searchpathlen = searchlen;

	boxwalk (fboxsize,fThresh);
	fillpoly (centerglobal.x, centerglobal.y);
	
	/* Kill everything below 75% down between the apexes */
	
/*
	dp (1, "Killing from %d to %d\n",c_apex1.y, c_apex2.y);
	for (y = (c_apex1.y+3*c_apex2.y)/4; y<=c_apex2.y; y++)
		for (x=0; x<spider_mask.xmax; x++)
			p[x + y*spider_mask.xmax]=0;

*/
	vert_visualize ();
}




/* vertebrae_main is directly linked to spiderop and displays the segmented areas */

void vertebrae_main (int searchlen, int fboxsize, float fThresh)
{


	searchpathlen = searchlen;
	ridgemax=0;

	/* Redisplay the secondary image, prepare to show segmented results.
	    Also, prepare a new image of the same size to hold the segmentation mask */
	
	if (allocate_image (&spider_mask, B_CHAR, spider_img.xmax, spider_img.ymax, spider_img.zmax)==-1)
		fatal_memerr();
		
	vertebrae_proc (searchlen, fboxsize, fThresh);
	
	/* Mask is no longer needed. Dispose */
	
	freebuf (&spider_mask);


}


/******************************* P A R T   2  ******************************************

	This part contains the measurement algorithms. It is assumed that spider_mask is
	properly set and can be used to mask the ROIs
	
*****************************************************************************************/


void compute_densities (char* s)
{
int x,y,i, xmax, ymax, stride;
double r, Rx,Cx,Sx, Rxx,Cxx,Sxx;
long rcnt, ccnt, scnt, idx, midx;
double savg,ssd, cavg,csd, ravg, rsd;
char* pmask;
char sdens[1024];
unsigned char *s_im_buf;

	xmax = gdk_pixbuf_get_width (s_img.pixbuf);
	ymax = gdk_pixbuf_get_height (s_img.pixbuf);
	stride = gdk_pixbuf_get_rowstride (s_img.pixbuf);
	s_im_buf = gdk_pixbuf_get_pixels (s_img.pixbuf);			/* Pointer to pixbuf's rgb data */

	pmask = spider_mask.data;
	Sx=Sxx=0; Cx=Cxx=0; Rx=Rxx=0;
	scnt=0; ccnt=0; rcnt=0;
	
	for (y=0; y<spider_img.ymax; y++)
		for (x=0; x<=spider_img.xmax; x++)
		{
			idx = x + spider_img.xmax*y;
			switch (pmask[idx])
			{
				case 1:		/* Spongiosa */
					r= readbuf_flt (&spider_img, x,y,0);
					Sx+=r; Sxx+=r*r; scnt++;
					break;
				case 2:		/* Cortical shell */
					r= readbuf_flt (&spider_img, x,y,0);
					Cx+=r; Cxx+=r*r; ccnt++;
					break;
				case 3:		/* Cortical ridge, is part of cortical shell */
					r= readbuf_flt (&spider_img, x,y,0);
					Cx+=r; Cxx+=r*r; ccnt++;
					if (y<=centerglobal.y)
					{
						Rx+=r; Rxx+=r*r; rcnt++;
					}
					break;
			}
		}
	
	if (scnt) savg=Sx/scnt; else savg=0;
	if (ccnt) cavg=Cx/ccnt; else cavg=0;
	if (rcnt) ravg=Rx/rcnt; else ravg=0;
	if (scnt>1) ssd = SDEV(Sx,Sxx,scnt); else ssd=0;
	if (ccnt>1) csd = SDEV(Cx,Cxx,ccnt); else csd=0;
	if (rcnt>1) rsd = SDEV(Rx,Rxx,rcnt); else rsd=0;
	
	sprintf (sdens, "Bone densities:\n   Spongiosa %6.1f +/- %6.1f (%li pixels)\n   cortex %6.1f +/- %6.1f (%li pixels)\n   ridge %6.1f +/- %6.1f (%li pixels)\n",
					savg, ssd, scnt, cavg, csd, ccnt, ravg, rsd, rcnt);
	strcat (s, sdens);


	/* Kludge. Convert the mask so that it outlines the cortical contour */
	
	for (y=0; y<spider_img.ymax; y++)
		for (x=0; x<=spider_img.xmax; x++)
		{
			idx = 3*x + stride*y;
			midx = x+spider_mask.xmax*y;
			s_im_buf[idx] = s_im_buf[idx+1] = s_im_buf[idx+2] = 0;
			if ((x>1) && (x<spider_img.xmax-2) && (y>1) && (y<spider_img.ymax-2))
				if ((pmask[midx]==0) || (pmask[midx]==1))	/* backgnd or spongiosa */
				{
					if ((pmask[midx-1]==2) || (pmask[midx+1]==2)
						|| (pmask[midx-spider_img.xmax]==2)
						|| (pmask[midx+spider_img.xmax]==2))
					{
						s_im_buf[idx]=0xff;
						s_im_buf[idx+1]=0xff;
						s_im_buf[idx+2]=0xff;
					}
				}
		}	

	secondary_image_redraw ();

}






/* vertebrae_meas performs a segmentation exactly like vertebrae_main, then computes various parameters for
   the bone */

void vertebrae_meas (int searchlen, int fboxsize, float fThresh)
{

	searchpathlen = searchlen;
	ridgemax=0;

	report = (char*) calloc (16384, sizeof (char));

	/* Redisplay the secondary image, prepare to show segmented results.
	    Also, prepare a new image of the same size to hold the segmentation mask */
	
	if (allocate_image (&spider_mask, B_CHAR, spider_img.xmax, spider_img.ymax, spider_img.zmax)==-1)
		fatal_memerr();
		
	vertebrae_proc (searchlen, fboxsize, fThresh);
	compute_densities (report);
	
	/* Mask is no longer needed. Dispose */
	
	freebuf (&spider_mask);
	
	ShowLongMessage (report, "Vertebral Data");
	
	free (report);

}


/********************************************************************

	B L O O D   V E S S E L   A N A L Y S I S
	
	This part is linked to Spiderop in a similar way as vertebrae 
	analysis. We assume that the segmented image is a blood vessel, 
	though, and analyze thickness etc.
	
**********************************************************************/


void analyze_vessel_slice (FILE* fp, int what, int searchlen, int fboxsize, float fThresh)
{
int i,j,xr,yr,cnt,xs,ys,xe,ye, xmax, ymax, stride;
long idx;
float val,max;
char brk;
float n, avg,len;
unsigned char *s_im_buf;


	xmax = gdk_pixbuf_get_width (s_img.pixbuf);
	ymax = gdk_pixbuf_get_height (s_img.pixbuf);
	stride = gdk_pixbuf_get_rowstride (s_img.pixbuf);
	s_im_buf = gdk_pixbuf_get_pixels (s_img.pixbuf);			/* Pointer to pixbuf's rgb data */


	/* Adjust rays to reflect local maximum
	   and set all ray elements related to local maximum */
	go_local_max (searchlen);
	
	for (i=0; i<numvertices; i++)	/* Run over all rays */
	{

		/* start at the local maximum then move outwards */
		max = rayforvertices[i].maxPixelValue;
		cnt=0; brk=0; n=0.0; avg=0.0;
		xs=ys=ye=xe=0;	/* to prevent a compiler warning */
		
		j=rayforvertices[i].maxPixelIndex;
		while ((j<rayforvertices[i].numPixels) && (brk<2))
		{
			if (++cnt <= fboxsize)		/* Limit absolute length */
			{
				xr = rayforvertices[i].XPixelCoords[j];
				yr = rayforvertices[i].YPixelCoords[j];
				val = readbuf_flt (&spider_img, xr+centerglobal.x, yr+centerglobal.y, 0);
				if ((val <= 0.9*max)	/* Let us drop away from the mandril */
				    && (val >= fThresh*max)) /* but within tissue density */
				{
				
					/* Determine average density and length */
					
					avg += val; n+=1;
					if (brk==0) 
					{
						brk=1;
						xs=xr; ys=yr;
					}
					xe=xr, ye=yr;
					
					/* visualize: color area lt blue */
					idx = 3*(xr+centerglobal.x) + stride*(yr+centerglobal.y);
					s_im_buf[idx++]=0;
					s_im_buf[idx++]=0x7f;
					s_im_buf[idx++]=0xff;
				}
				else
					if (brk==1) brk=2;	/* make sure the line is continuous */
			}
			j++;

		}
		if (n>0) avg /= n; else avg=0.0;	/* Average density */
		len = sqrt ((float)SQR(xe-xs)+(float)SQR(ye-ys));	/* thickness */
		
		if (fp)
		{
			switch (what)
			{
				case 0:
					fprintf (fp, "%8.2f ", avg);
					break;
				case 1:
					fprintf (fp, "%8.2f ", len);
					break;
			}
		}
		

	}
	if (fp) fprintf (fp, "\n");

	secondary_image_redraw ();

}


/* The driver to analyze a single slice of vessel data */

void vessel_meas (int searchlen, int fboxsize, float fThresh)
{

	analyze_vessel_slice (NULL, 1, searchlen, fboxsize, fThresh);

}

/* The driver to run over the entire length of the vessel */


void vessel_meas_all (int searchlen, int fboxsize, float fThresh)
{
int z,saveslice;
FILE* fp;

	saveslice = cur_slice;
	
	fp = fopen ("vessel.txt","wt");
	if (!fp) return;
	
	for (z=10; z<mainimg.zmax; z++)
	{
		make_imlib_buffer_from_main_image (z);
		if (zslice_callback)
				zslice_callback ();	/* points back to spiderop */
				
		analyze_vessel_slice (fp,1, searchlen, fboxsize, fThresh);
	}
	fclose (fp);

	cur_slice = saveslice;
	make_imlib_buffer_from_main_image (cur_slice);
	if (zslice_callback)
			zslice_callback ();	/* points back to spiderop */

}
